' ===========================================================================
' FILE:  INET_Display.vb
'
' ----------------- D O C U  M E N T A T I O N ---------------------------
'
' This iNet Interface file contains code to manage displays in visual basic.
' A "Display" is a PictureBox on the front panel that can show a variable
' number of waveforms, and is VERY programmable.
'
' -------------------------  H I S T O R Y --------------------------------
' HISTORY  Copyright c 1994-98 GWI.  All rights reserved.
' 7/29/98  created - gsw
' 6/6/02   updated to be compatible with Visual Studio (Visual Basic) .Net v7
' ===========================================================================

Option Strict Off
Option Explicit On 
Imports System.Drawing.Printing             ' contains printing support
Imports System.Drawing.Drawing2D

Module Display_Utilities

    ' ===========================================================================
    ' PRIMARY GRAPHICS OBJECTS
    ' ===========================================================================

    ' End user's main window FORM, gsw 6/1/02
    Public gEndUser_FORM As System.Windows.Forms.Form

    ' GRAPHICS object used in end user's window, gsw 6/1/02
    Public gEndUser_GRAPHICS As System.Drawing.Graphics

    ' Visual Basic PRINTER object, gsw 6/1/02
    Public gEndUser_PRINTER As System.Drawing.Printing.PrintDocument

    ' EndUser_Window is a CLASS created by the end user that contains
    ' a window (i.e. Visual Basic FORM) and several callback functions:
    ' EndUser_SegmentProcessor(), EndUser_SaveLoadResults(), and
    ' EndUser_PostAcqAnalysis()
    Public gEndUser_CLASS As EndUser_Window

    ' Pen used for drawing in end user's window
    Public gEndUser_PEN As System.Drawing.Pen

    ' Pen used for drawing in end user's window
    Public gEndUser_FONT As System.Drawing.Font

    ' ===========================================================================
    ' CONSTANTS AND GLOBALS -- used with plotting displays
    ' ===========================================================================

    ' max # of channels per displaybox
    Public Const MAX_NUM_CH_PER_OneBox As Short = 32

    ' max # of Display Boxes
    Public Const MAX_NUM_OneBox As Short = 32

    ' dNum for top display in DispBox
    Public Const TOP_DISPLAY_NUM As Short = 0

    ' # of displays boxes
    Public gNumBoxes As Short

    ' automatically select grid spacing
    Public Const AUTO_GRID As Short = -1

    ' # of pixels per grid when doing AUTO_GRID (each grid is 50/72th = .6" high)
    Public Const crtPix_per_grid_AUTO_GRID As Short = 50

    ' no previous marker position
    Public Const NO_PREVIOUS_MKR As Short = -1

    ' # of free pixels below & above label text
    Public Function vSPACE_yPix(ByRef g_form As System.Drawing.Graphics) As Short
        CrtPix_to_GraphicsPix(3, g_form)    ' 1/25th of an inch, 3 screen pixels
    End Function

    ' max # of markers
    Public Const MAX_NUM_MKRS As Short = 4

    ' no scroll bar
    Public Const NO_SCROLL As Short = -1

    ' Display is not linked to a digitize channel
    Public Const NO_DIGI_CHAN As Double = -1.0#

    ' return string (Chr$(13) + Chr(10))
    Public gReturnStr As String

    ' -----------------------------------------------------
    ' Line sytles types.

    Public Const kBlack As Short = 1            ' SCREEN: solid black   PRINTER: solid black
    Public Const kGray As Short = 2             ' SCREEN: solid gray    PRINTER: mesh
    Public Const kBlue As Short = 3             ' SCREEN: solid blue    PRINTER: solid black
    Public Const kRed As Short = 4              ' SCREEN: solid red     PRINTER: solid black
    Public Const kWhite As Short = 5            ' SCREEN: solid white   PRINTER: solid white

    ' -----------------------------------------------------
    ' Display types.

    Public Const kYindexDisplay As Short = 1    ' Y vs index display
    Public Const kXYDisplay As Short = 2        ' X vs Y display

    ' -----------------------------------------------------
    ' Automatically add channels that have been enabled
    ' for digitizing in the instruNet World NETWORK window.
    ' This is done when the DisplayBox is added to the window, when
    ' you close the instruNet World Network window and when you Load in data.

    Public Const hAutoAddNone As Short = 1      ' don't automatically do anythin
    Public Const hAutoAddTimeWaves As Short = 2 ' automatically add time plots of digitized waves
    Public Const hAutoAddFreqWaves As Short = 3 ' automatically add fft's of digitized waves
    Public Const hAutoAddXYplots As Short = 4   ' automatically add xy plots of digitized waves

    ' -----------------------------------------------------
    ' Parameters for 1 channel (multiple channels within one display box).

    ' turns vPos/hPos methods on/off
    Public gMethod_vPos_scrollObj_Enabled As Short
    Public gMethod_hPos_scrollObj_Enabled As Short


    ' ===========================================================================
    ' STRUCT    DispParm
    '
    ' PURPOSE   Contains parameters for 1 Display (1 Channel per Display)
    '           that resides within a Multi-Display DisplayBox.
    ' ===========================================================================
    Public Structure DispParm

        ' --------------------------------------
        ' HORIZONTAL SCALE PARAMS

        Dim dispLeft_xEU As Double          ' horiz Left/Right values in xEU coords
        Dim dispRight_xEU As Double
        Dim dispWidth_xEU As Double
        Dim xCoord_Per_xEU As Double        ' xCoords per xEU (horiz plot scale)

        ' --------------------------------------
        ' Y vs INDEX PLOT PARAMS

        Dim dispLeft_ptNum As Integer       ' waveform ptNum corresponding to left edge
        Dim dispRight_ptNum As Integer      ' waveform ptNum corresponding to right
        Dim dispWidth_numPts As Integer     ' width of display in points
        Dim xCoord_Per_ptNum As Double      ' xCoords per points (horiz plot scale)

        ' --------------------------------------
        ' VERTICAL SCALE PARAMS

        Dim dispTop_yEu As Double           ' top of display, yEU units
        Dim dispBot_yEu As Double           ' bottom of display, yEU units
        Dim dispMid_yEu As Double           ' middle of display, yEU units
        Dim dispHeight_yEu As Double        ' height of display, yEU units
        Dim yCoord_Per_yEU As Double        ' yCoord per yEU (yCoord to yEU translation)
        Dim plotOffset_yEU As Double        ' plot offset, yEU units
        Dim plotOffset_yCoord As Double     ' plot offset, yCoord units

        ' --------------------------------------
        ' X/Y DISPLAY COORDINATES

        Dim dispLeft_xCoord As Short        ' left of display, xCoord units  (i.e. "scale", not pixels)
        Dim dispRight_xCoord As Short       ' right of display, xCoord units
        Dim dispTop_yCoord As Short         ' top of display, yCoord units
        Dim dispBot_yCoord As Short         ' bottom of display, yCoord units
        Dim dispHeight_yCoord As Short      ' Height/Width - calc'ed from top/bot/lef/rht
        Dim dispWidth_xCoord As Short

        ' --------------------------------------
        ' GRID COORDINATES

        Dim showGrid As Short               ' if true, we show a grid
        Dim numGridLines_horiz As Short     ' # of horiz grid lines, e.g. 3 (0 to turn off grid, -1= autoselect)
        Dim numGridLines_vert As Short      ' # of vertical grid lines, e.g. 5 (0 to turn off grid, -1= autoselect)
        Dim ticksPerGrid_vert As Short      ' # of tics per vert grid, e.g. 3 (-1= autoselect)
        Dim ticksPerGrid_horiz As Short     ' # of tics per horiz grid, e.g. 3 (-1= autoselect)

        ' Calculated, based on the above values

        Dim numGridLines_horiz_calc As Short
        Dim numGridLines_vert_calc As Short
        Dim ticksPerGrid_vert_calc As Short
        Dim ticksPerGrid_horiz_calc As Short
        Dim gridWidth_xCoord As Double      ' space between vertical grid lines
        Dim gridHeight_yCoord As Double     ' space between horiz grid lines

        ' --------------------------------------
        ' DIGTIIZE DATA

        ' index into gDigiChan(0..numChan) array to get to data associated
        ' w/ this display. if this = NO_DIGI_CHAN = -1, then use the LOCAL data below.
        Dim cNum As Short

        ' --------------------------------------
        ' LOCAL DATA

        ' Display data is kept in dataArrayY, if the source is not the gDigiChan(cNum).dataArrayY,
        ' which is the data directly from a digitization. If this is the case, then .cNum = NO_DIGI_CHAN
        '
        ' dataArrayY Contains (for Y data):
        '
        '   .numValidPts = # of valid points contained within the data() buffer
        '   .numBufferPts = data() buffer size
        '   .data() = data is stored in this array
        '   .ptsPerXunit = sample rate of the data
        '   .firstPt_Xunits = first point time in seconds
        '
        Dim dataArrayY As DataArrayType

        ' dataArray for X data, when plotting X vs Y (i.e. displayType = kXYDisplay)
        Dim dataArrayX As DataArrayType

        ' --------------------------------------
        ' HOUSEKEEPING

        Dim clrBeforePlot As Short  ' if true, we need to clear the display right
        '                             before plotting, the next time we plot
        Dim name As String          ' name of Display
        Dim displayType As Short    ' type of display = {kYindexDisplay, kXYDisplay}

        ' --------------------------------------
        ' MARKERS

        ' wave value at the marker position
        <VBFixedArray(MAX_NUM_MKRS)> Dim valueAtMarker_yEU() As Double

        ' initialize instances of this structure, gsw 6/1/02
        Public Sub Initialize()
            ReDim valueAtMarker_yEU(MAX_NUM_MKRS)
        End Sub

    End Structure


    ' ===========================================================================
    ' STRUCT    DispBoxParm
    '
    ' PURPOSE   Contains parameters that define a iNet DisplayBox (which 
    '           can show 1 to 32 channels in one box).
    ' ===========================================================================
    Public Structure DispBoxParm

        ' --------------------------------------
        ' ENCLOSING WINDOW-BASED OBJECTS

        ' box object on front panel that encloses all displays in this 1 DispBox.
        Dim BoxObj As System.Windows.Forms.PictureBox

        ' window that contains DispBox
        Dim FormObj As System.Windows.Forms.Form

        ' GRAPHICS object that is used to direct all
        ' drawing commands to this Display box
        ' Not needed now: Public box_GRAPHICS As System.Drawing.Graphics

        ' --------------------------------------
        ' HORIZONTAL SCALE

        Dim hPos_scrollObj As System.Windows.Forms.HScrollBar   ' Horiz scrollbar, = "Nothing" if doesn't exist
        Dim hPos_scroll_lastValue As Short                      ' Horiz scrollbar last value

        ' --------------------------------------
        ' VERTICAL SCALE

        Dim vPos_scrollObj As System.Windows.Forms.VScrollBar   ' Vert scrollbar, = "Nothing" if doesn't exist
        Dim vPos_scroll_lastValue As Short                      ' Vert scrollbar last value

        ' --------------------------------------
        ' MARKERS
        '
        ' Current marker position, in waveform points
        ' this is the dNum of the display within the
        ' dispBox that holds the marker (the marker refers
        ' to the value of the wave in this 1 display).

        <VBFixedArray(MAX_NUM_MKRS)> Dim markerPos_xEU() As Double
        <VBFixedArray(MAX_NUM_MKRS)> Dim markerPos_dNum() As Short

        ' Previous marker position (we need to replot wave here),
        ' display of previous marker, = NO_PREVIOUS_MKR if none

        <VBFixedArray(MAX_NUM_MKRS)> Dim prevMarkerPos_xEU() As Double
        <VBFixedArray(MAX_NUM_MKRS)> Dim prevMarkerPos_dNum() As Short

        Dim numMkrs As Short                    ' # of markers in the display
        Dim supportMarkers As Short             ' if True, this dispBox supports markers; otherwise not

        ' --------------------------------------
        ' X/Y COORDINATES

        Dim dispBoxLeft_xCoord As Short         ' left of dispBox, xCoord units (i.e. "scale", not pixels)
        Dim dispBoxRight_xCoord As Short        ' right of dispBox, xCoord units
        Dim dispBoxTop_yCoord As Short          ' top of dispBox, yCoord units
        Dim dispBoxBot_yCoord As Short          ' bottom of dispBox, yCoord units
        Dim vLabelBoxLeft_xCoord As Short       ' this is the left of the Vertical Label region and

        ' Top of the Lower Horizontal Label region at the bottom of the DisplayBox 
        ' (and below the bottom-most Display)
        Dim hLowerLabel_Top_yCoord As Short

        ' Bottom of the Upper Horizontal Label region at the top of the DisplayBox
        ' (and above the top-most Display)
        Dim hUpperLabel_Bot_yCoord As Short

        ' --------------------------------------
        ' GRID

        Dim supportGrid_default As Short        ' true if you draw grid
        Dim numGridLines_horiz_default As Short ' # of horiz grid lines, e.g. 3 (0 to turn off grid)
        Dim numGridLines_vert_default As Short  ' # of vertical grid lines, e.g. 5 (0 to turn off grid)
        Dim ticksPerGrid_vert_default As Short  ' # of tics per vert grid, e.g. 3
        Dim ticksPerGrid_horiz_default As Short ' # of tics per horiz grid, e.g. 3

        ' --------------------------------------
        ' MANAGEMENT

        ' Automatically add DigiChan() channels to DisplayBox.  This is done when the
        ' DisplayBox is added to the window, when you close the instruNet World Network window
        ' and when you Load in data.
        Dim autoAddPolicy As Short

        Dim hDispBoxTitle_text As String       ' text at top of DisplayBox, that labels the box
        Dim numDisp As Short                   ' # of displays in DispBox (i.e. # of channels)
        Dim disp() As DispParm                 ' contains params for all Displays within the DispBox 
        '                                       (we expand this each time we add a display)

        ' initialize instances of these arrays, gsw 6/1/02
        Public Sub Initialize()
            ReDim markerPos_xEU(MAX_NUM_MKRS)
            ReDim markerPos_dNum(MAX_NUM_MKRS)
            ReDim prevMarkerPos_xEU(MAX_NUM_MKRS)
            ReDim prevMarkerPos_dNum(MAX_NUM_MKRS)
        End Sub

    End Structure

    ' ===========================================================================
    ' GLOBAL VARIABLES - used with plotting displays
    ' ===========================================================================

    ' Contains all the DispBox params (the size of this array is ReDim'ed
    ' after initializing each DispBox).
    Public gBox() As DispBoxParm

    ' ===========================================================================
    ' CONSTANTS - used with plotting displays
    ' ===========================================================================

    ' -----------------------------------------------------
    ' Flags passed to Reset_Controls() to tell it what to reset

    Public Const reset_hPosition_toZero As Short = 1
    Public Const reset_hScale_toOneScanWideDisplays As Short = 2
    Public Const reset_hScaleToInternalValues As Short = 4
    Public Const reset_ptsPerHscale As Short = 8
    Public Const reset_vScalePosScrollBtns As Short = 16
    Public Const reset_vScaleToInternalValues As Short = 32
    Public Const reset_mkrs_leftEdge As Short = 64

    ' ===========================================================================
    ' ===========================================================================
    '
    '                                   SUBROUTINES
    '
    ' ===========================================================================
    ' ===========================================================================

    ' -------------------------------------------------------------------------
    ' FUNCTION    NumToStr()
    '
    ' PURPOSE     Convert a number to string
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function NumToStr(ByVal num As Double) As String

        Dim numAbs As Double
        numAbs = System.Math.Abs(num)

        If (numAbs = 0) Then
            NumToStr = "0"
        ElseIf (numAbs < 0.001) Then
            NumToStr = VB6.Format(num, "scientific") ' e.g. 3.23E-02
        ElseIf (numAbs < 0.01) Then
            NumToStr = VB6.Format(num, ".######")
        ElseIf (numAbs < 0.1) Then
            NumToStr = VB6.Format(num, ".#####")
        ElseIf (numAbs < 1.0#) Then
            NumToStr = VB6.Format(num, ".####")
        ElseIf (numAbs < 10.0#) Then
            NumToStr = VB6.Format(num, "0.###")
        ElseIf (numAbs < 100.0#) Then
            NumToStr = VB6.Format(num, "##.##")
        ElseIf (numAbs < 1000.0#) Then
            NumToStr = VB6.Format(num, "###.#")
        ElseIf (numAbs < 10000.0#) Then
            NumToStr = VB6.Format(num, "####.#")
        ElseIf (numAbs < 100000.0#) Then
            NumToStr = VB6.Format(num, "#####.#")
        Else
            NumToStr = VB6.Format(num, "scientific") ' e.g. 3.23E+02
        End If

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    Reset_Controls_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Reset horiz/vert scroll/scale to default values, per 'thingsToReset':
    '
    '                   Global Const reset_hPosition_toZero = 1
    '                   Global Const reset_hScale_toOneScanWideDisplays = 2
    '                   Global Const reset_hScaleToInternalValues = 4
    '                   Global Const reset_ptsPerHscale = 8
    '                   Global Const reset_vScalePosScrollBtns = 16
    '                   Global Const reset_vScaleToInternalValues = 32
    '                   Global Const reset_mkrs_leftEdge = 64
    '       
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub Reset_Controls_OneBox(ByVal bNum As Short, ByRef thisBox As DispBoxParm, _
        ByRef thingsToReset As Short, ByRef g_form As System.Drawing.Graphics)

        Dim dispRight_xEU_NEW, OneScanWide_TopDisplay_xEU As Double
        Dim mkrNum, dNum, resetLeftEdgeToZero, cNum, reset_hScale_toOneScanWide As Short
        Dim dispLeft_xEU_NEW, dispMid_xEU_NEW, dispMid_xEU_current, sRate As Double
        Dim ptsPerXunit_topDisp As Double
        Dim numValidPts_topDisp As Integer
        Dim firstPt_Xunits_topDisp, ptsPerXunit, firstPt_Xunits As Double
        Dim numValidPts As Integer
        Dim bot, top, leftEdge, rightEdge As Double

        ' ---------------------------------------------------
        ' We will work on one DisplayBox

        With thisBox

            ' Don't do anything in here if there are no displays in this DisplayBox.
            If (.numDisp <= 0) Then Exit Sub

            ' Get the # of data pts and the data's sample rate, for the TOP display.
            Call GetDataParams(bNum, TOP_DISPLAY_NUM, ptsPerXunit_topDisp, _
                numValidPts_topDisp, firstPt_Xunits_topDisp)

            ' ---------------------------------------------------
            ' HORIZONTAL SCALE (i.e. xEU Left/Right parameters).

            ' if we are to reset the hPosition to 0...
            resetLeftEdgeToZero = False
            If (thingsToReset And reset_hPosition_toZero) <> 0 Then

                If ValidObject(.hPos_scrollObj) Then

                    ' turn off hPos_Change() method for a moment if this is Y vs INDEX, then reset left edge to 0
                    gMethod_hPos_scrollObj_Enabled = False

                    If (.disp(TOP_DISPLAY_NUM).displayType = kYindexDisplay) Then
                        .hPos_scrollObj.Value = thisBox.hPos_scrollObj.Minimum
                    Else
                        .hPos_scrollObj.Value = .hPos_scrollObj.Minimum + (((.hPos_scrollObj.Maximum - .hPos_scrollObj.LargeChange + 1) - .hPos_scrollObj.Minimum) * 0.5)
                    End If

                    .hPos_scroll_lastValue = .hPos_scrollObj.Value
                    gMethod_hPos_scrollObj_Enabled = True

                End If

                resetLeftEdgeToZero = True

            End If

            ' if we are to reset the hScale to set displays 1scan wide...
            reset_hScale_toOneScanWide = IIf(((thingsToReset And reset_hScale_toOneScanWideDisplays) <> 0), True, False)

            ' reset the horiz scale control setting to 1 scan per display width
            If (reset_hScale_toOneScanWide = True) Or (resetLeftEdgeToZero = True) Then

                ' Calc width of 1 scan for the top display.
                OneScanWide_TopDisplay_xEU = numValidPts_topDisp / ptsPerXunit_topDisp

                ' Make sure this is ok.
                If OneScanWide_TopDisplay_xEU < 1.0E-30 Then OneScanWide_TopDisplay_xEU = 1

                ' Set the scale for all displays.
                For dNum = 0 To (.numDisp - 1)

                    With .disp(dNum)

                        ' if this is Y vs INDEX, then reset left edge to 0
                        If (.displayType = kYindexDisplay) Then

                            dispLeft_xEU_NEW = IIf(resetLeftEdgeToZero = True, 0, .dispLeft_xEU)
                            dispRight_xEU_NEW = IIf((reset_hScale_toOneScanWide = True), (dispLeft_xEU_NEW + (OneScanWide_TopDisplay_xEU)), (dispLeft_xEU_NEW + (.dispRight_xEU - .dispLeft_xEU)))

                            ' else, if this is X vs Y, then reset center to 0
                        Else

                            dispMid_xEU_current = .dispLeft_xEU + (.dispRight_xEU - .dispLeft_xEU) / 2
                            dispMid_xEU_NEW = IIf(resetLeftEdgeToZero = True, 0, dispMid_xEU_current)

                            If (reset_hScale_toOneScanWide = True) Then
                                dispLeft_xEU_NEW = dispMid_xEU_NEW - OneScanWide_TopDisplay_xEU / 2
                                dispRight_xEU_NEW = dispMid_xEU_NEW + OneScanWide_TopDisplay_xEU / 2
                            Else
                                dispLeft_xEU_NEW = dispMid_xEU_NEW - .dispWidth_xEU / 2
                                dispRight_xEU_NEW = dispMid_xEU_NEW + .dispWidth_xEU / 2
                            End If

                        End If

                        ' force update by making it different
                        .dispLeft_xEU = dispRight_xEU_NEW + 1

                        ' left and right display edges
                        Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, _
                            dispLeft_xEU_NEW, dispRight_xEU_NEW, g_form)

                    End With

                Next dNum

            End If

            ' ---------------------------------------------------
            ' SET HORIZONTAL SCALE TO .dispLeft_xEU/.dispRight_xEU values

            ' if reset the vertical scale to ..dispLeft_xEU/.dispRight_xEU values...
            If (thingsToReset And reset_hScaleToInternalValues) <> 0 Then

                ' Set hScroll bar to it's lastValue
                If ValidObject(.hPos_scrollObj) Then ' if we have a scroll bar...
                    gMethod_hPos_scrollObj_Enabled = False
                    .hPos_scrollObj.Value = .hPos_scroll_lastValue
                    gMethod_hPos_scrollObj_Enabled = True
                End If

                ' Scan through all Displays in the Box...
                For dNum = 0 To (.numDisp - 1)

                    leftEdge = .disp(dNum).dispLeft_xEU
                    rightEdge = .disp(dNum).dispRight_xEU

                    ' force update by making it different
                    .disp(dNum).dispLeft_xEU = leftEdge + 0.001

                    ' set vScale to .dispLeft_xEU/.dispRight_xEU values
                    Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, _
                        leftEdge, rightEdge, g_form)

                Next dNum

            End If

            ' ---------------------------------------------------
            ' SET Pts-per-hSCALE based on the sample rate of
            ' the data in the data's 'ptsPerSec' field
            ' (i.e. set the ptNum-vs-xEU/xCoord stuff).

            ' if we are to reset the hPosition to 0...
            If (thingsToReset And reset_ptsPerHscale) <> 0 Then

                ' Scan through all Displays in the Box...
                For dNum = 0 To (.numDisp - 1)

                    ' get digitize channel that is linked to this display
                    cNum = .disp(dNum).cNum

                    ' get the # of data pts and the data's sample rate (this display)
                    Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)

                    Call hScale_SetPtsPerHscale_OneDisp(bNum, dNum, ptsPerXunit)

                Next dNum

            End If

            ' ---------------------------------------------------
            ' SET VERTICAL SCALE TO +/-5V

            ' if reset the vertical scale to +/-5V...
            If (thingsToReset And reset_vScalePosScrollBtns) <> 0 Then

                ' Scan through all Displays in the Box...
                For dNum = 0 To (.numDisp - 1)

                    ' force update by making it different
                    .disp(dNum).dispBot_yEu = 0.001

                    ' set vScale to -5V/+5V default
                    Call vScale_SetWithNew_yEU_values_OneDisp(bNum, dNum, 5, -5, g_form)

                Next dNum

                ' Put vScroll bar in center
                If ValidObject(.vPos_scrollObj) Then ' if we have a scroll bar...

                    gMethod_vPos_scrollObj_Enabled = False
                    .vPos_scrollObj.Value = .vPos_scrollObj.Minimum + _
                        (((.vPos_scrollObj.Maximum - .vPos_scrollObj.LargeChange + 1) - _
                        .vPos_scrollObj.Minimum) * 0.5)
                    .vPos_scroll_lastValue = .vPos_scrollObj.Value
                    gMethod_vPos_scrollObj_Enabled = True

                End If

            End If

            ' ---------------------------------------------------
            ' SET VERTICAL SCALE TO .dispTop_yEU/.dispBox_yEU values

            ' if reset the vertical scale to .dispTop_yEU/.dispBox_yEU values...
            If (thingsToReset And reset_vScaleToInternalValues) <> 0 Then

                ' Scan through all Displays in the Box...
                For dNum = 0 To (.numDisp - 1)

                    ' Put vScroll bar to it's last value
                    If ValidObject(.vPos_scrollObj) Then ' if we have a scroll bar...

                        gMethod_vPos_scrollObj_Enabled = False
                        .vPos_scrollObj.Value = .vPos_scroll_lastValue
                        gMethod_vPos_scrollObj_Enabled = True

                    End If

                    bot = .disp(dNum).dispBot_yEu
                    top = .disp(dNum).dispTop_yEu

                    ' force update by making it different
                    .disp(dNum).dispBot_yEu = bot + 0.001

                    ' set vScale to .dispTop_yEU/.dispBox_yEU values
                    Call vScale_SetWithNew_yEU_values_OneDisp(bNum, dNum, top, bot, g_form)

                Next dNum

            End If

            ' ---------------------------------------------------
            ' RESET MARKERS

            ' if reset the markers to the left edge...
            If (thingsToReset And reset_mkrs_leftEdge) <> 0 Then

                ' move markers to left edge
                For mkrNum = 0 To (.numMkrs - 1)

                    .markerPos_xEU(mkrNum) = .disp(TOP_DISPLAY_NUM).dispLeft_xEU
                    .prevMarkerPos_xEU(mkrNum) = .disp(TOP_DISPLAY_NUM).dispLeft_xEU

                    .markerPos_dNum(mkrNum) = TOP_DISPLAY_NUM
                    .prevMarkerPos_dNum(mkrNum) = NO_PREVIOUS_MKR ' no previous marker at this time

                Next mkrNum

            End If

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_vPos_scroll_KeyDown()
    '
    ' PURPOSE     Handle a key down in the vPos scroll bar
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_vPos_scroll_KeyDown(ByVal bNum As Short, _
        ByVal KeyCode As Short, ByVal Shift As Short)

        With gBox(bNum)

            ' don't do anything in here if there are not displays in this DisplayBox.
            If (.numDisp <= 0) Then Exit Sub

            ' if the scrollbar does not exist, bail out
            If Not (ValidObject(.vPos_scrollObj)) Then Exit Sub

            ' reset to left edge
            If KeyCode = &H30S Then

                ' reset vScroll to it's center
                .vPos_scrollObj.Value = .vPos_scrollObj.Minimum + _
                    (((.vPos_scrollObj.Maximum - .vPos_scrollObj.LargeChange + 1) _
                    - .vPos_scrollObj.Minimum) * 0.5)

                ' update the vScroll bar
                METHOD_vPos_scroll_Change(bNum)

            End If
        End With
    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Add_One_DisplayBox_To_Window()
    '
    ' PURPOSE     Create and Initialize one dispBox. There must be a PictureBox
    '             object on the front panel that corresponds to the box that we
    '             are creating.
    '
    '             DisplayBoxes are PictureBox rectangles on a form that are set up
    '             by the end user at design time using the VB toolbar.  They are
    '             indexed controls (i.e. set index property to 0, 1...) and
    '             must be named "Box" (i.e. set name property to "Box").  The
    '             code handles any number of these arrayed controls, and indentifies
    '             each with a 'bNum' which goes from 0 to the (numBoxes-1).
    '             This number corresponds to the index of the Picture box on the form.
    '             After the user has placed these "Boxes", the code must know they
    '             are there and this is done with Add_One_DisplayBox_To_Window().  This is called
    '             for each "Box" on the front panel.  One must pass the name of the
    '             box object, the form object, and optionally horiz/vertical scrollbars.
    '             If there are no scrollbars, please pass "Nothing".
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub Add_One_DisplayBox_To_Window( _
        ByRef BoxObj_Panel As System.Windows.Forms.PictureBox, _
        ByRef hPos_scroll_Panel As System.Windows.Forms.HScrollBar, _
        ByRef vPos_scroll_Panel As System.Windows.Forms.VScrollBar, _
        ByRef FormObj_Panel As System.Windows.Forms.Form, _
        ByVal hDispBoxTitle_text As String, _
        ByRef autoAddPolicy As Short, _
        ByVal g_form As System.Drawing.Graphics)

        ' -------------------------------------------------------------
        ' Local Variables

        Dim NewLargeChange, bNum As Integer     ' 32bit integer
        Dim enclosingRectF As RectangleF

        ' -------------------------------------------------------------
        ' Create RAM to hold the params for this new Display.

        gNumBoxes = gNumBoxes + 1       ' increment # of DisplayBoxes

        ReDim Preserve gBox(gNumBoxes)  ' make the gBox array one larger

        bNum = gNumBoxes - 1            ' work w/ last, new, box

        ' -------------------------------------------------------------
        ' Initialize.

        With gBox(bNum)

            ' -------------------------------------------------------------
            ' Update the objects.

            .FormObj = FormObj_Panel    ' window containing DispBox
            .BoxObj = BoxObj_Panel      ' PictureBox object on Form

            ' GRAPHICS object that is used to direct all
            ' drawing commands to this Display box
            '       Not needed now: .box_GRAPHICS = .BoxObj.CreateGraphics()

            ' We're going to hide the boxObj for now, so we can draw to the Form,
            ' at the BoxObj position, and see the drawings.
            .BoxObj.Visible = False

            ' Initialize instances of markerPos_xEU(), markerPos_dNum(), 
            ' prevMarkerPos_xEU(), prevMarkerPos_dNum(), gsw 6/1/02
            .Initialize()

            ' scrollbars, = "Nothing" if not there
            .hPos_scrollObj = hPos_scroll_Panel
            .vPos_scrollObj = vPos_scroll_Panel

            ' set up scrollbar to help with creating even numbers
            If ValidObject(.hPos_scrollObj) Then

                gMethod_hPos_scrollObj_Enabled = False
                .hPos_scrollObj.Value = 0
                .hPos_scrollObj.Minimum = 0
                .hPos_scrollObj.Maximum = (30000 + .hPos_scrollObj.LargeChange - 1)
                .hPos_scrollObj.SmallChange = 3000
                NewLargeChange = 6000
                .hPos_scrollObj.Maximum = .hPos_scrollObj.Maximum + NewLargeChange - .hPos_scrollObj.LargeChange
                .hPos_scrollObj.LargeChange = NewLargeChange
                gMethod_hPos_scrollObj_Enabled = True

            End If

            ' set up scrollbar to help with creating even numbers
            If ValidObject(.vPos_scrollObj) Then

                gMethod_vPos_scrollObj_Enabled = False
                .vPos_scrollObj.Value = 15000
                .vPos_scrollObj.Minimum = 0
                .vPos_scrollObj.Maximum = (30000 + .vPos_scrollObj.LargeChange - 1)
                .vPos_scrollObj.SmallChange = 1500
                NewLargeChange = 6000
                .vPos_scrollObj.Maximum = .vPos_scrollObj.Maximum + NewLargeChange - .vPos_scrollObj.LargeChange
                .vPos_scrollObj.LargeChange = NewLargeChange
                gMethod_vPos_scrollObj_Enabled = True

            End If

            ' -------------------------------------------------------------
            ' Initialize fundamental parameters. If you don't like any of these,
            ' you can change them after calling this Initialize routine.

            ' text at top of DisplayBox, that labels the box
            .hDispBoxTitle_text = hDispBoxTitle_text

            ' no markers in the DispBox at this time
            .numMkrs = 0

            ' don't support markers for now -- if one wants them, they must call Add_One_Marker_To_OneBox().
            .supportMarkers = False

            ' this DispBox contains no Displays at this time (we add them later)
            .numDisp = 0

            ' automatically add DigiChan() channels to DisplayBox
            .autoAddPolicy = autoAddPolicy

            ' -------------------------------------------------------------
            ' Initialize the default grid line space for each DisplayBox.
            ' The following params are for each Display inside this DisplayBox.

            ' # of horiz grid lines, e.g. 3 (0 to turn off grid, -1 = autoselect)
            .numGridLines_horiz_default = AUTO_GRID

            ' # of vertical grid lines, e.g. 5 (0 to turn off grid, -1 = autoselect)
            .numGridLines_vert_default = AUTO_GRID

            ' # of tics per vert grid, e.g. 5 (-1 = autoselect)
            .ticksPerGrid_vert_default = AUTO_GRID

            ' # of tics per horiz grid, e.g. 3 (-1 = autoselect)
            .ticksPerGrid_horiz_default = AUTO_GRID

            ' support drawing a grid
            .supportGrid_default = True

            ' -------------------------------------------------------------
            ' ReCalculate DisplayBox's x/y coordinates.

            ' Get the x/y coords of the enclosing rect for this displayBox
            ' (e.g. VB picture box object on the VB Form, or Printer page)
            enclosingRectF = DisplayBox_Get_EnclosingRectangle(bNum, g_form)

            ' ReCalculate DisplayBox's x/y position coordinates, given an enclosing rectangle
            DisplayBox_Calculate_Given_EnclosingRect(bNum, enclosingRectF, g_form)

            ' -------------------------------------------------------------
            ' If we are to automatically add channels to display, then do so now.

            ' if we are automatically add channels to this box...
            If (.autoAddPolicy <> hAutoAddNone) Then

                ' Automatically add Digitize Channels to this display box.
                '
                ' This first Deletes all Displays in this DisplayBox before
                ' it adds the channels to the box, one channel per Display
                ' in this one DisplayBox.
                '
                ' This will set the display data() buffers to PtsPerScan size.
                AutoAddChannelsToThisDisplayBox(bNum, g_form)

            End If

            ' -------------------------------------------------------------
            ' Reset the DispBox horiz/vertical position/scale controls

            Call Reset_Controls_OneBox(bNum, gBox(bNum), 255, g_form)

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DisplayBox_Calculate_Given_EnclosingRect()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Calculated the DisplayBox's x/y position coordinates, given
    '             an enclosing rectangle (e.g. VB picture box object on the VB Form,
    '             or Printer page).
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub DisplayBox_Calculate_Given_EnclosingRect( _
        ByRef bNum As Short, _
        ByRef enclosingRectangleF As RectangleF, _
        ByVal g_form As System.Drawing.Graphics)

        With gBox(bNum)

            ' calc text hight in pixel units
            Dim TextHeight As Integer
            TextHeight = g_form.MeasureString( _
                (.hDispBoxTitle_text & "A"), gEndUser_FONT).Height

            ' calculate height of label area (one above, and one below the plot region)
            Dim labelBoxHeight As Integer = (TextHeight + 2 * (vSPACE_yPix(g_form)))

            ' get enclosing rectangle for DisplayBox.
            .dispBoxLeft_xCoord = enclosingRectangleF.Left
            .dispBoxRight_xCoord = enclosingRectangleF.Right
            .dispBoxTop_yCoord = enclosingRectangleF.Top
            .dispBoxBot_yCoord = enclosingRectangleF.Bottom

            ' Don't allow DisplayBox Size to go negative (else code gets into trouble).
            ' If this code makes this modification, the displays will overlap and look
            ' messay, yet at least it the code will run.
            Dim dispBoxBot_yCoord_MIN = .dispBoxTop_yCoord + ((labelBoxHeight * 2) + CrtPix_to_GraphicsPix(10, g_form))
            If .dispBoxBot_yCoord < dispBoxBot_yCoord_MIN Then .dispBoxBot_yCoord = dispBoxBot_yCoord_MIN
            If .dispBoxRight_xCoord < (.dispBoxLeft_xCoord + 60) Then .dispBoxRight_xCoord = (.dispBoxLeft_xCoord + 60)

            ' put left edge of vertical label box 100 pixels from right edge of DisplayBox
            .vLabelBoxLeft_xCoord = .dispBoxRight_xCoord _
                - (g_form.MeasureString("-8.88E-88", gEndUser_FONT).Width + CrtPix_to_GraphicsPix(10, g_form))

            ' put lower Horizontal Label region at the 20 lowest pixels in the DisplayBox
            .hLowerLabel_Top_yCoord = .dispBoxBot_yCoord - labelBoxHeight

            ' put bottom of the Upper Horizontal Label region at the 20pix below the
            ' top of the DisplayBox (and above the top-most Display)
            .hUpperLabel_Bot_yCoord = .dispBoxTop_yCoord + labelBoxHeight

            ' show alert if display height gets too low
            If (.hLowerLabel_Top_yCoord < (.hUpperLabel_Bot_yCoord + 3)) Then MsgBox("DisplayBox too small.")

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    LastDisplay_Initialize()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Initializes the last Display in DisplayBox # 'bNum'.  
    '             'displayType' is the type of display = {kYindexDisplay, kXYDisplay}.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub LastDisplay_Initialize( _
        ByVal bNum As Short, _
        ByVal displayType As Short, _
        ByRef g_form As System.Drawing.Graphics)

        ' -------------------------------------------------------------
        ' Get the dNum of the last display (which is the one we are initializing).

        Dim mkrNum
        Dim dNum_newDisplay As Integer = gBox(bNum).numDisp - 1

        ' -------------------------------------------------------------
        ' Set the display type (y, xy).

        gBox(bNum).disp(dNum_newDisplay).displayType = displayType

        ' -------------------------------------------------------------
        ' Initialize MARKER's for this new display

        With gBox(bNum).disp(dNum_newDisplay)

            ' Initialize the .valueAtMarker_yEU() array to be MAX_NUM_MKRS long
            gBox(bNum).disp(dNum_newDisplay).Initialize()

            ' clear the yEU wave values at each marker
            For mkrNum = 0 To (MAX_NUM_MKRS - 1)
                .valueAtMarker_yEU(mkrNum) = 0
            Next mkrNum

        End With

        ' -------------------------------------------------------------
        ' Calculate a display's x/y coordinates.

        DisplayBox_Calculate_Enclosed_Displays(bNum, g_form)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DisplayBox_Calculate_Enclosed_Displays()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Calculate a display's x/y coordinates.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub DisplayBox_Calculate_Enclosed_Displays( _
        ByVal bNum As Short, _
        ByRef g_form As System.Drawing.Graphics)

        ' -------------------------------------------------------------
        ' Local Variables.

        Dim plotRegionHeight_yCoord, dispHeight_pixel As Double
        Dim dNum_newDisplay, dNum, mkrNum As Short
        Dim top_pixel, bot_pixel As Integer

        ' -------------------------------------------------------------
        ' Reposition all plot & labels Region within the DisplayBox.

        With gBox(bNum)

            plotRegionHeight_yCoord = .hLowerLabel_Top_yCoord - .hUpperLabel_Bot_yCoord
            dispHeight_pixel = plotRegionHeight_yCoord / .numDisp
            top_pixel = .hUpperLabel_Bot_yCoord

        End With

        ' -------------------------------------------------------------
        ' set pixel coords for plotRegion for each Display in DispBox.

        For dNum = 0 To (gBox(bNum).numDisp - 1)

            With gBox(bNum).disp(dNum)

                ' Take same left and right pixel positions as DispBox itself,
                ' minus the Vertical label region at the right.
                .dispLeft_xCoord = gBox(bNum).dispBoxLeft_xCoord
                .dispRight_xCoord = gBox(bNum).vLabelBoxLeft_xCoord

                ' Don't allow Display Size to go negative (else code crashes)
                If .dispRight_xCoord < (.dispLeft_xCoord + 30) Then .dispRight_xCoord = (.dispLeft_xCoord + 30)

                ' Take same left and right pixel positions as DispBox itself.
                bot_pixel = ((dNum + 1) * dispHeight_pixel) + gBox(bNum).hUpperLabel_Bot_yCoord

                ' Assign plotRegion top/bottom pixel coords.
                .dispBot_yCoord = bot_pixel
                .dispTop_yCoord = top_pixel

                ' Don't allow Display Size to go negative (else code crashes)
                If .dispBot_yCoord < (.dispTop_yCoord + 10) Then .dispBot_yCoord = .dispTop_yCoord + 10

                ' Bottom becomes the new top.
                top_pixel = bot_pixel

                ' Calc display height and width.
                .dispHeight_yCoord = .dispBot_yCoord - .dispTop_yCoord
                .dispWidth_xCoord = .dispRight_xCoord - .dispLeft_xCoord

                ' Establishes the vertical/horiz grid for this display.
                Call Grid_SetSpacing(gBox(bNum), gBox(bNum).disp(dNum), _
                    gBox(bNum).supportGrid_default, gBox(bNum).numGridLines_horiz_default, _
                    gBox(bNum).numGridLines_vert_default, gBox(bNum).ticksPerGrid_vert_default, _
                    gBox(bNum).ticksPerGrid_horiz_default, g_form)

            End With

        Next dNum

        ' -------------------------------------------------------------
        ' Reset the DispBox horiz/vertical position/scale controls to default value.
        '
        ' Sets the following HORIZONTAL SCALE params:
        '
        '   .xEU left/right
        '   .ptNum left/right
        '
        ' Sets the following VERTICAL SCALE params:
        '
        '   .yCoord_Per_yEU     yCoord per yEU
        '   .plotOffset_yEU     plot offset, yEU units
        '   .dispHeight_yEu     height of display, yEU units
        '
        '   .dispTop_yEu        top of display, yEU units
        '   .dispBot_yEu        bottom of display, yEU units
        '   .dispMid_yEu        middle of display, yEU units

        Call Reset_Controls_OneBox(bNum, gBox(bNum), _
            reset_hPosition_toZero + reset_hScale_toOneScanWideDisplays + _
            reset_ptsPerHscale + reset_vScalePosScrollBtns + reset_mkrs_leftEdge, g_form)

    End Sub

    ' -------------------------------------------------------------------------
    ' FUNCTION    DisplayBox_All_Recalculate()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Recalculates All x/y coordinates for All Displays within 
    '             All DisplayBoxes.  This is useful after end user has
    '             Resized the main window (i.e. VB Form).
    '           
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub DisplayBox_All_Recalculate(ByRef g_form As System.Drawing.Graphics)

        Dim bNum As Short

        ' if there are no displays, or the window is hidden, then exit
        If DisplayBoxes_Are_Not_Visible(g_form) Then Exit Sub

        ' scan through all DisplayBoxes
        For bNum = 0 To gNumBoxes - 1

            ' Recalculates All x/y coordinates for 1 DisplayBox 
            ' and it's enclosing Displays.             
            DisplayBox_Calculate_DisplayBox_And_All_Displays(bNum, g_form)

        Next bNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DisplayBox_Calculate_Box_And_Displays()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Recalculates All x/y coordinates for 1 DisplayBox and
    '             it's enclosing Displays.  
    '           
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub DisplayBox_Calculate_DisplayBox_And_All_Displays(ByVal bNum As Short, _
        ByRef g_form As System.Drawing.Graphics)

        Dim enclosingRectF As RectangleF

        ' Get the x/y coords of the enclosing rect for this displayBox
        ' (e.g. VB picture box object on the VB Form, or Printer page)
        enclosingRectF = DisplayBox_Get_EnclosingRectangle(bNum, g_form)

        ' ReCalculate DisplayBox's x/y position coordinates, given an enclosing rectangle
        DisplayBox_Calculate_Given_EnclosingRect(bNum, enclosingRectF, g_form)

        ' ReCalculate All Display x/y coords given the 1 enclosing DisplayBox
        DisplayBox_Calculate_Enclosed_Displays(bNum, g_form)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DisplayBox_All_Recalculate()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Get the x/y coords of the enclosing rect for this displayBox
    '             (e.g. VB picture box object on the VB Form, or Printer page)
    '           
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function DisplayBox_Get_EnclosingRectangle( _
        ByVal bNum As Short, _
        ByRef g_form As System.Drawing.Graphics) As RectangleF

        ' if there are no displays, or the window is hidden, then exit
        If DisplayBoxes_Are_Not_Visible(g_form) Then Exit Function

        Dim enclosingRectF As RectangleF

        ' if we are drawing to a printer page
        If DrawingTo_PRINTER(g_form) Then

            ' the enclosing rectangle is one printed page. In this case,
            ' gEndUser_GRAPHICS is a GRAPHICS port from the PRINTER device.
            enclosingRectF = gEndUser_GRAPHICS.VisibleClipBounds()

            ' Else we are drawing to the screen (and use the VB PictureBox
            ' object
        Else

            ' the enclosing rectangle is one printed page. In this case,
            ' gEndUser_GRAPHICS is a GRAPHICS port from the PRINTER device.
            enclosingRectF = New RectangleF( _
                gBox(bNum).BoxObj.Left, gBox(bNum).BoxObj.Top, _
                gBox(bNum).BoxObj.Width, gBox(bNum).BoxObj.Height)

        End If

        ' return the rectangle that encloses this display box
        DisplayBox_Get_EnclosingRectangle = enclosingRectF

    End Function

    ' -------------------------------------------------------------------------
    ' FUNCTION    DisplayBoxes_Are_Not_Visible()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Returns True if displayBoxes are not present, or window is hidden;
    '             False otherwise
    '           
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function DisplayBoxes_Are_Not_Visible(ByRef g_form As System.Drawing.Graphics) As Boolean

        DisplayBoxes_Are_Not_Visible = True

        ' if we have no display boxes
        If gNumBoxes <= 0 Then Exit Function

        If IsNothing(g_form) Then Exit Function

        If g_form.VisibleClipBounds.Width < 100 Then Exit Function

        If (g_form.IsVisible(50, 50) = False) Then Exit Function

        DisplayBoxes_Are_Not_Visible = False

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    Add_One_Marker_To_OneBox()
    '
    ' PURPOSE     Add one vertical marker line to the DisplayBox.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Add_One_Marker_To_OneBox(ByVal bNum As Short)

        With gBox(bNum)

            ' Record that this display box support markers
            .supportMarkers = True

            ' Marker position, in xEU coordinates
            .markerPos_xEU(.numMkrs) = 0.0#

            ' This is the dNum of the display within the dispBox that holds
            ' the marker (the marker refers to the value of the wave in this 1 display).
            .markerPos_dNum(.numMkrs) = TOP_DISPLAY_NUM

            ' previous marker position
            .prevMarkerPos_xEU(.numMkrs) = 0
            .prevMarkerPos_dNum(.numMkrs) = NO_PREVIOUS_MKR

            ' # markers in the DispBox at this time
            .numMkrs = .numMkrs + 1

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Grid_SetSpacing()
    '
    ' PURPOSE     Set the grid spacing for 1 Display.  If these value are set to
    '             AUTO_GRID = -1, then the code selects something automatically
    '             that is pleasing to the eye.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Grid_SetSpacing(ByRef thisBox As DispBoxParm, ByRef thisDisp As DispParm, _
        ByVal supportGrid As Short, ByVal numGridLines_horiz As Short, _
        ByVal numGridLines_vert As Short, ByVal ticksPerGrid_vert As Short, _
        ByVal ticksPerGrid_horiz As Short, ByRef g_form As System.Drawing.Graphics)

        With thisDisp

            ' # of horiz grid lines, e.g. 3 (0 to turn off grid, -1 = autoselect)
            .numGridLines_horiz = numGridLines_horiz

            ' # of vertical grid lines, e.g. 5  (0 to turn off grid, -1 = autoselect)
            .numGridLines_vert = numGridLines_vert

            ' # of tics per vert grid, e.g. 3 (-1 = autoselect)
            .ticksPerGrid_vert = ticksPerGrid_vert

            ' # of tics per horiz grid, e.g. 3 (-1 = autoselect)
            .ticksPerGrid_horiz = ticksPerGrid_horiz

            ' if true, support drawing a grid
            .showGrid = supportGrid

            ' -----------------------------------------------------
            ' Calc grid params if doing AUTO_GRID (let computer decide grid spacing).

            .numGridLines_horiz_calc = .numGridLines_horiz
            If .numGridLines_horiz = AUTO_GRID Or .numGridLines_horiz = 0 Then

                ' Calc # of horiz grid lines in 1 Display (each grid is 50/72th = .6" high)
                .numGridLines_horiz_calc = .dispHeight_yCoord / CrtPix_to_GraphicsPix(crtPix_per_grid_AUTO_GRID, g_form)

                ' The display looks better if there is an odd # of lines,
                ' so that we have a zero line.
                If (.numGridLines_horiz_calc / 2) = Int((.numGridLines_horiz_calc + 1) / 2) Then
                    .numGridLines_horiz_calc = .numGridLines_horiz_calc + 1
                End If

            End If

            .numGridLines_vert_calc = .numGridLines_vert
            If .numGridLines_vert = AUTO_GRID Or .numGridLines_vert = 0 Then

                ' Calc # of vert grid lines in 1 Display (each grid is 50/72th = .6" high)
                .numGridLines_vert_calc = .dispWidth_xCoord / CrtPix_to_GraphicsPix(crtPix_per_grid_AUTO_GRID, g_form)

                ' The display looks better if there is an odd # of lines,
                ' so that we have a zero line.
                If (.numGridLines_vert_calc / 2) = Int((.numGridLines_vert_calc + 1) / 2) Then
                    .numGridLines_vert_calc = .numGridLines_vert_calc + 1
                End If

            End If

            .ticksPerGrid_vert_calc = .ticksPerGrid_vert
            If .ticksPerGrid_vert = AUTO_GRID Or .ticksPerGrid_vert = 0 Then
                .ticksPerGrid_vert_calc = 3
            End If

            .ticksPerGrid_horiz_calc = .ticksPerGrid_horiz
            If .ticksPerGrid_horiz = AUTO_GRID Or .ticksPerGrid_horiz = 0 Then
                .ticksPerGrid_horiz_calc = 3
            End If

            ' -----------------------------------------------------
            ' Calc x/y space between each grid.

            .gridWidth_xCoord = (.dispRight_xCoord - .dispLeft_xCoord) / (.numGridLines_vert_calc + 1)
            .gridHeight_yCoord = (.dispBot_yCoord - .dispTop_yCoord) / (.numGridLines_horiz_calc + 1)

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    AutoAddChannelsToThisDisplayBox()
    '
    ' PURPOSE     Automatically adds digitize channels to this display box.
    '
    '             We first Delete all Displays in this DisplayBox before
    '             we add the channels to the box, one channel per Display
    '             in this one DisplayBox.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub AutoAddChannelsToThisDisplayBox(ByVal bNum As Short, _
        ByRef g_form As System.Drawing.Graphics)

        With gBox(bNum)

            ' if this DisplayBox has AutoAdd turned off, then bail out
            If (.autoAddPolicy = hAutoAddNone) Then Exit Sub

            ' -------------------------------------------------------------
            ' Start off with no channels this box.

            ' We first Delete all Displays in this DisplayBox before
            ' we add the channels to the box, one channel per Display
            ' in this one DisplayBox.

            .numDisp = 0
            ReDim Preserve .disp(0)

            ' -------------------------------------------------------------
            ' Add all channels in the gDigiChan() list to this DisplayBox.

            ' if there are not digitize channels, then bail out
            If (gNumDigiCh <= 0) Then Exit Sub

            ' if Automatically add time plots of digitized waves...
            If (.autoAddPolicy = hAutoAddTimeWaves) Then

                Call Add_DigitizeChannels_To_OneBox(bNum, 0, gNumDigiCh - 1, kYindexDisplay, g_form)

                ' else if Automatically add fft's of digitized waves...
            ElseIf (.autoAddPolicy = hAutoAddFreqWaves) Then

                Call Add_DigitizeChannels_To_OneBox(bNum, 0, gNumDigiCh - 1, kXYDisplay, g_form)

                ' else if automatically add xy plots of digitized waves...
            ElseIf (.autoAddPolicy = hAutoAddXYplots) Then

                MsgBox("hAutoAddXYplots Not yet coded.")
                EndProgram_iNet((""))

            End If

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Add_DigitizeChannels_To_OneBox()
    '
    ' PURPOSE     Add channels gDigiChan('cNum_top') to gDigiChan('cNum_bot')
    '             to DisplayBox # 'bNum'.  gDigiChan(..) channels are defined with
    '             DigitizeList_AddOneItem().
    '
    '             Example: Supppose we created 6 channels with DigitizeList_AddOneItem()
    '             and we created 2 DisplayBoxes with Add_One_DisplayBox_To_Window(), and we want to
    '             put the 1st 3 channels into 1 box, and the later 3 channels into the
    '             2nd box.  We would then call:
    '
    '               Add_DigitizeChannels_To_OneBox(0=dispBoxNum, 0=digiChNum_top, 2=digiChanNum_bot)
    '               Add_DigitizeChannels_To_OneBox(1=dispBoxNum, 3=digiChNum_top, 5=digiChanNum_bot)
    '
    '            'displayType' is the type of display = {kYindexDisplay, kXYDisplay}.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Add_DigitizeChannels_To_OneBox( _
        ByVal bNum As Short, _
        ByVal cNum_top As Short, _
        ByVal cNum_bot As Short, _
        ByVal displayType As Short, _
        ByRef g_form As System.Drawing.Graphics)

        ' -------------------------------------------------------------
        ' Local Variables

        Dim dNum_newDisplay, cNum, dNum As Short

        ' we operate on this one display box
        With gBox(bNum)

            ' we scan through the digitize channels that are being added to this one display box
            For cNum = cNum_top To cNum_bot

                ' -------------------------------------------------------------
                ' Create RAM to hold the params for this new Display.

                ' inc the # of Displays in this DispBox
                .numDisp = .numDisp + 1

                dNum_newDisplay = .numDisp - 1

                ' Make the disp() array larger. This array contains params for each display.
                ReDim Preserve .disp(.numDisp)

                ' -------------------------------------------------------------
                ' Attach the new display to the digitize data buffers.

                With .disp(dNum_newDisplay)

                    ' attach the display to the digitize channel.
                    .cNum = cNum

                    ' Initialize instances valueAtMarker_yEU(), gsw 6/1/02
                    .Initialize()

                    ' since this display is linked to a digitize channel,
                    ' then set its local data to nil
                    .dataArrayY.ptsPerXunit = 0
                    .dataArrayY.firstPt_Xunits = 0
                    .dataArrayY.numValidPts = 0
                    .dataArrayY.numBufferPts = 0

                    .dataArrayY.gcHandle = Nothing        ' handle to data() array, after being pinned
                    .dataArrayY.pinnedAddress = 0         ' address to data() array, after being pinned
                    .dataArrayY.arrayIsPinned = False     ' true if data() has been pinned

                    ' set data() array size to 0
                    ReDim .dataArrayY.data(0)

                    ' we set the X array to nil as well (only used w/ XY plots)
                    .dataArrayX.ptsPerXunit = 0
                    .dataArrayX.firstPt_Xunits = 0
                    .dataArrayX.numValidPts = 0
                    .dataArrayX.numBufferPts = 0

                    .dataArrayX.gcHandle = Nothing        ' handle to data() array, after being pinned
                    .dataArrayX.pinnedAddress = 0         ' address to data() array, after being pinned
                    .dataArrayX.arrayIsPinned = False     ' true if data() has been pinned

                    ReDim .dataArrayX.data(0)

                End With

                ' give the Display the name of the channel
                Call iNet_VB_GetFieldcStr(gDigiChan(cNum), sgt_General, _
                    fldNum_General_userName, .disp(dNum_newDisplay).name)

                ' attach the digiChan to the display
                gDigiChan(dNum_newDisplay).bNum = bNum
                gDigiChan(dNum_newDisplay).dNum = dNum_newDisplay

                ' -------------------------------------------------------------
                ' Initialize this new display

                Call LastDisplay_Initialize(bNum, displayType, g_form)

            Next cNum

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Add_NonDigitizeDisplay_To_OneBox()
    '
    ' PURPOSE     Adds a non-digitize channel to a display. Non-digitize means
    '             that the code does not automatically place a digitized waveform
    '             into the display.  Data is stored in
    '
    '                gBox(bNum).disp(dNum).dataArrayY.data(ptNum)
    '
    '             where bNum is this DisplayBox index, dNum is the Display index
    '             within the box, an ptNum is the point index within the data array.
    '             The following holds important information about the data:
    '
    '                gBox(bNum).disp(dNum).dataArrayY.ptsPerXunit - data sample rate
    '                gBox(bNum).disp(dNum).dataArrayY.numBufferPts - data array size
    '                gBox(bNum).disp(dNum).dataArrayY.numValidPts - # of valid data pts
    '                gBox(bNum).disp(dNum).dataArrayY.data() - actual data array
    '
    ' PARAMS      bNum - DisplayBox index in the main window = {0...gNumBoxes-1}
    '             displayType - type of display = {kYindexDisplay, kXYDisplay}.
    '             numValidPts - number of valid pts in the data array
    '             numBufferPts - data array size
    '             ptsPerXunit - data array sample rate, in pts per horizontal x units
    '                   This could be < 1 (e.g. 256 points of 1KHz fft data will yield
    '                   .25 pts per Hz.
    '             name - name of the Display
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Add_NonDigitizeDisplay_To_OneBox( _
        ByVal bNum As Short, _
        ByVal displayType As Short, _
        ByVal numValidPts As Integer, _
        ByVal numBufferPts As Integer, _
        ByVal ptsPerXunit As Double, _
        ByVal name As String, _
        ByRef g_form As System.Drawing.Graphics)

        ' -------------------------------------------------------------
        ' Local Variables

        Dim ptNum, last_ptNum, dNum_newDisplay As Integer

        ' -------------------------------------------------------------
        ' Create RAM to hold the params for this new Display.

        ' inc the # of Displays in this DispBox
        gBox(bNum).numDisp = gBox(bNum).numDisp + 1

        dNum_newDisplay = gBox(bNum).numDisp - 1

        ' set disp() array size to the # of Displays
        ' Old: ReDim Preserve gBox().disp(gBox(bNum).numDisp), gsw 6/1/02
        With gBox(bNum)
            ReDim Preserve .disp(gBox(bNum).numDisp)
        End With

        ' with this display box ...
        With gBox(bNum).disp(dNum_newDisplay)

            ' -------------------------------------------------------------
            ' Set up local data buffer.

            ' we are NOT attached to a digitize channel
            .cNum = NO_DIGI_CHAN

            ' sample rate of the local_data(), in pts-per-plotted-xUnit
            .dataArrayY.ptsPerXunit = ptsPerXunit

            ' first point time in seconds
            .dataArrayY.firstPt_Xunits = 0

            ' # of valid points contained within the local_data() buffer.
            .dataArrayY.numValidPts = numValidPts

            ' data() buffer size
            .dataArrayY.numBufferPts = numBufferPts

            .dataArrayY.gcHandle = Nothing        ' handle to data() array, after being pinned
            .dataArrayY.pinnedAddress = 0         ' address to data() array, after being pinned
            .dataArrayY.arrayIsPinned = False     ' true if data() has been pinned

            ' create ram to hold the data, add 2 for saftey
            ReDim .dataArrayY.data(numBufferPts + 2)

            ' zero the data
            last_ptNum = numValidPts - 1
            For ptNum = 0 To last_ptNum
                .dataArrayY.data(ptNum) = 0
            Next ptNum

            ' if doing Y vs index Display...
            If (displayType = kYindexDisplay) Then

                ' set the X array to nil, since we only use the Y array
                .dataArrayX.ptsPerXunit = 1
                .dataArrayX.firstPt_Xunits = 0
                .dataArrayX.numValidPts = 0
                .dataArrayX.numBufferPts = 0

                ' set dataArrayX data() length to 0 points
                ReDim .dataArrayX.data(.dataArrayX.numBufferPts)

                ' else we are doing xy displays, and need to set up the X data array
            Else

                ' setup the internal x axis data array the same as the y array
                .dataArrayX.ptsPerXunit = .dataArrayY.ptsPerXunit
                .dataArrayX.firstPt_Xunits = .dataArrayY.firstPt_Xunits
                .dataArrayX.numValidPts = .dataArrayY.numValidPts
                .dataArrayX.numBufferPts = .dataArrayY.numBufferPts

                ' set dataArrayX data() length to many points
                ReDim .dataArrayX.data(.dataArrayX.numBufferPts)

            End If

            .dataArrayX.gcHandle = Nothing        ' handle to data() array, after being pinned
            .dataArrayX.pinnedAddress = 0         ' address to data() array, after being pinned
            .dataArrayX.arrayIsPinned = False     ' true if data() has been pinned

            ' display name
            .name = name

        End With

        ' -------------------------------------------------------------
        ' Initialize this new display.

        Call LastDisplay_Initialize(bNum, displayType, g_form)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    ValidObject()
    '
    ' PURPOSE     Returns true if this is a valid object that has been "set"
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function ValidObject(ByRef obj As Object) As Short

        Dim hasValue, validObj As Short

        ' we assume the object is not valid in some way, and 
        ' if it passes the following tests, we'll say it's valid
        validObj = False

        ' if we hit an error, jump to ErrorHandler
        On Error GoTo ErrorHandler

        ' test object several different ways
        If IsNothing(obj) Then GoTo ErrorHandler
        If IsDBNull(obj) Then GoTo ErrorHandler
        If Not (IsReference(obj)) Then GoTo ErrorHandler

        ' Not used in VS 7: if it Error's on the following line, then the obj might not be set
        ' Not used in VS 7: hasValue = obj

        ' since we passed these tests, we can now say the obj is valid
        validObj = True

ErrorHandler:

        ' if we hit an error, there is a problem and the
        ' object is probably not there

        ValidObject = validObj

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    vScale_ChangeScale_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             This changes the scale by a scaleMult, and then we update
    '             the vertical scale (dispTop_xEU, dispBot_xEU, etc).
    '
    ' PARAMS        INPUT    scaleMult, yCoord_Per_yEU, etc
    '
    '               OUTPUT  dispHeight_yEu
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub vScale_ChangeScale_OneBox(ByVal bNum As Short, ByVal scaleMult As Double, _
        ByRef g_form As System.Drawing.Graphics)

        Dim dNum As Short
        Dim dispHeight_yEU_NEW, dispTop_yEu_NEW, dispBot_yEu_NEW As Double

        ' don't do anything in here if there are no displays in this DisplayBox.
        If (gBox(bNum).numDisp <= 0) Then Exit Sub

        ' go thru each Display in this Box
        For dNum = 0 To gBox(bNum).numDisp - 1

            ' with this Display...
            With gBox(bNum).disp(dNum)

                ' calc new height w/ the scaleMult
                dispHeight_yEU_NEW = .dispHeight_yEu * scaleMult

                ' set the display top/bottom given the new height
                dispTop_yEu_NEW = .dispMid_yEu + (dispHeight_yEU_NEW / 2)
                dispBot_yEu_NEW = .dispMid_yEu - (dispHeight_yEU_NEW / 2)

                ' set the new vScale
                Call vScale_SetWithNew_yEU_values_OneDisp(bNum, dNum, dispTop_yEu_NEW, dispBot_yEu_NEW, g_form)

            End With

        Next dNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    hScale_StretchScale_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             This changes the scale by a scaleMult, and then we update
    '             the horizontal scale (dispLeft_xEU, dispRight_xEU, etc).
    '
    ' PARAMS        INPUT    scaleMult, xCoord_Per_xEU, etc
    '
    '               OUTPUT  dispWidth_xEu
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub hScale_StretchScale_OneBox(ByVal bNum As Short, ByVal scaleMult As Double, _
        ByRef g_form As System.Drawing.Graphics)

        Dim dNum As Short
        Dim dispWidth_xEU_NEW, dispLeft_xEU_NEW, dispRight_xEU_NEW As Double

        ' don't do anything in here if there are no displays in this DisplayBox.
        If (gBox(bNum).numDisp <= 0) Then Exit Sub

        ' go thru each Display in this Box
        Dim dispMid_xEU As Double
        For dNum = 0 To gBox(bNum).numDisp - 1

            ' with this Display...
            With gBox(bNum).disp(dNum)

                ' calc new width w/ the scaleMult
                dispWidth_xEU_NEW = .dispWidth_xEU * scaleMult

                ' if xy display...
                If (.displayType = kXYDisplay) Then

                    ' expand out from the center
                    dispMid_xEU = ((.dispRight_xEU - .dispLeft_xEU) / 2) + .dispLeft_xEU
                    dispLeft_xEU_NEW = dispMid_xEU - (dispWidth_xEU_NEW / 2)
                    dispRight_xEU_NEW = dispMid_xEU + (dispWidth_xEU_NEW / 2)

                    ' else if y vs index display...
                Else

                    ' expand from left edge
                    dispLeft_xEU_NEW = .dispLeft_xEU
                    dispRight_xEU_NEW = dispLeft_xEU_NEW + dispWidth_xEU_NEW

                End If

                ' set the new vScale
                Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, _
                    dispLeft_xEU_NEW, dispRight_xEU_NEW, g_form)

            End With

        Next dNum

    End Sub

    ' .
    ' -------------------------------------------------------------------------
    ' FUNCTION    vScale_SetWithNew_yEU_values_OneDisp()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Given a new dispTop_yEU/dispBot_yEU, we update the vertical scale
    '             (dispTop_xEU, dispBot_xEU, etc) for 1 Display.
    '
    ' PARAMS        INPUT   dispTop_yEU, dispBot_yEU
    '
    '               OUTPUT  dispMid_yEu, dispHeight_yEu, yCoord_Per_yEU, plotOffset_yEU
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub vScale_SetWithNew_yEU_values_OneDisp(ByVal bNum As Short, ByVal dNum As Short, _
        ByVal dispTop_yEu As Double, ByVal dispBot_yEu As Double, _
        ByRef g_form As System.Drawing.Graphics)

        ' don't do anything in here if there are no displays in this DisplayBox.
        If (gBox(bNum).numDisp <= 0) Then Exit Sub

        With gBox(bNum).disp(dNum)

            .dispTop_yEu = dispTop_yEu      ' top of display, yEU units
            .dispBot_yEu = dispBot_yEu      ' bottom of display, yEU units

            .dispHeight_yEu = dispTop_yEu - dispBot_yEu         ' height of display, yEU units
            .dispMid_yEu = .dispBot_yEu + (.dispHeight_yEu / 2) ' middle of display, yEU units

            ' yCoord per yEU (used w/ yCoord to yEU translation)
            .yCoord_Per_yEU = (.dispBot_yCoord - .dispTop_yCoord) / (.dispBot_yEu - .dispTop_yEu)

            ' Debugging:
            '   a = .plotOffset_yEU: b = .dispBot_yCoord: c = .yCoord_Per_yEU: d = .dispBot_yEu

            ' yCoord = (yEU + .plotOffset_yEU ) * .yCoord_Per_yEU
            .plotOffset_yEU = (.dispBot_yCoord / .yCoord_Per_yEU) - .dispBot_yEu

            ' yEU = (yCoord + .plotOffset_yCoord ) / .yCoord_Per_yEU
            .plotOffset_yCoord = (.dispBot_yEu * .yCoord_Per_yEU) - .dispBot_yCoord

            ' For debugging:
            '   temp = .dispBot_yCoord: tempB = .dispTop_yCoord: tempC = .dispHeight_yEu
            '   tempD = .yCoord_Per_yEU: tempE = .plotOffset_yCoord: tempF = .plotOffset_yEU

            ' update the vertical scale labels
            Call vScale_DrawLabel_OneDisp(bNum, dNum, gBox(bNum).disp(dNum), g_form)

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    vScale_DrawLabel_OneDisp()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Draws vScale labels to right of display.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub vScale_DrawLabel_OneDisp(ByVal bNum As Short, ByVal dNum As Short, _
        ByRef thisDisp As DispParm, ByRef g_form As System.Drawing.Graphics)

        Dim yEU, yCoord_double As Double
        Dim yEU_str As String
        Dim textHeight_yCoord, textWidth_xCoord, yCoord_int, i, _
            Pen_yCoord, Pen_xCoord As Integer
        Dim needToPrintName As Short
        needToPrintName = True

        ' 4 crt pixes (approx .05")
        Dim vLABEL_SPACE As Short = CrtPix_to_GraphicsPix(4, g_form)

        With gBox(bNum)

            ' don't do anything in here if there are not displays in this DisplayBox.
            If (.numDisp <= 0) Then Exit Sub

            Call Set_AutoRedraw_Property(True, g_form)

            ' -----------------------------------------------------
            ' fill the entire vLabel region w/ white

            Clear_Rectangle( _
                (.vLabelBoxLeft_xCoord + 1), _
                (thisDisp.dispTop_yCoord + 1), _
                (.dispBoxRight_xCoord - 1), _
                (thisDisp.dispBot_yCoord - 1), g_form)

            ' -----------------------------------------------------
            ' Draw vScale (e.g. "0   .1   .2   .3   .4 secs" vertically,
            ' one label w/ each grid line

            ' select black line style
            Call Set_gEndUser_PEN(kBlack, g_form)

            ' get the height of the TEXT
            textHeight_yCoord = gEndUser_FONT.GetHeight(g_form)

            For i = 0 To (thisDisp.numGridLines_horiz_calc + 1)

                ' calc yCoord of vertical line
                yCoord_double = thisDisp.dispBot_yCoord - (thisDisp.gridHeight_yCoord * i)
                yCoord_int = yCoord_double

                ' get yEU value for this grid line

                ' yEU = (yCoord + .plotOffset_yCoord ) / .yCoord_Per_yEU
                yEU = (yCoord_double + thisDisp.plotOffset_yCoord) / thisDisp.yCoord_Per_yEU

                '  if the display height is > .1, then round off to nearest .001
                If thisDisp.dispHeight_yEu > 0.1 Then
                    yEU = Int((yEU * 1000.0#) + 0.5) / 1000.0#
                End If

                yEU_str = NumToStr(yEU)

                ' if this is the center line, then print the channel name
                If (i = Int((.disp(dNum).numGridLines_horiz_calc / 2) + 0.51)) And needToPrintName Then
                    yEU_str = .disp(dNum).name
                    needToPrintName = False
                End If

                If i = 0 Then '                 if we are at the bottom, then push up
                    Pen_yCoord = yCoord_int - (textHeight_yCoord + (vLABEL_SPACE / 2))
                    '                           if we are at the top, push down
                ElseIf (i = (.disp(dNum).numGridLines_horiz_calc + 1)) Then
                    Pen_yCoord = yCoord_int + (vLABEL_SPACE / 2)
                Else
                    Pen_yCoord = yCoord_int - (textHeight_yCoord / 2)
                End If

                ' set the upper left coord of the printed label
                Pen_xCoord = .vLabelBoxLeft_xCoord + vLABEL_SPACE

                ' print the xEU scale value
                g_form.DrawString(yEU_str, gEndUser_FONT, Brushes.Black, Pen_xCoord, Pen_yCoord)
NextGrid:
            Next i

            Call Set_AutoRedraw_Property(False, g_form)

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Clear_Rectangle()
    '
    ' PURPOSE     Clears the Rectangle (i.e. fills with white).
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Clear_Rectangle(ByVal x1 As Integer, ByVal y1 As Integer, _
        ByVal x2 As Integer, ByVal y2 As Integer, ByRef g_form As System.Drawing.Graphics)

        ' create a white brush
        Dim brushColor As New SolidBrush(Color.White)

        ' fill the rectangle with white
        g_form.FillRectangle(brushColor, _
            IIf(x1 < x2, x1, x2), _
            IIf(y1 < y2, y1, y2), _
            Math.Abs(x2 - x1), _
            Math.Abs(y2 - y1))

        brushColor.Dispose()

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_vPos_increment_Click()
    '
    ' PURPOSE     Increment vertical position of display.  Invoked by a Click
    '             on vPos_increment button.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_vPos_increment_Click(ByVal bNum As Short)

        ' don't do anything in here if there are not displays in this DisplayBox.
        If (gBox(bNum).numDisp <= 0) Then Exit Sub

        ' dec vertical scale of DispBox
        Call vScale_ChangeScale_OneBox(bNum, 2, gEndUser_GRAPHICS)

        ' redraw 1 DispBox
        Call Redraw_OneBox(bNum, gEndUser_GRAPHICS)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_vPos_decrement_Click()
    '
    ' PURPOSE     Decrement vertical position of display.  Invoked by a Click
    '             on vPos_decrement button.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_vPos_decrement_Click(ByVal bNum As Short)

        ' don't do anything in here if there are not displays in this DisplayBox.
        If (gBox(bNum).numDisp <= 0) Then Exit Sub

        ' dec vertical scale of DispBox
        Call vScale_ChangeScale_OneBox(bNum, 0.5, gEndUser_GRAPHICS)

        ' redraw 1 DispBox
        Call Redraw_OneBox(bNum, gEndUser_GRAPHICS)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_vPos_scroll_Change()
    '
    ' PURPOSE     Update vertical position of a display.  Invoked by the change 
    '             of the Vertical Scroll bar.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_vPos_scroll_Change(ByVal bNum As Short)

        Dim dNum As Integer
        Dim deltaScrollValue_Normalized, delta_yEU As Double
        Const Scroll_Range As Short = 4

        ' we will operate on one Display box
        With gBox(bNum)

            ' don't do anything in here if there are not displays in this DisplayBox.
            If (.numDisp <= 0) Then Exit Sub

            ' if the scrollbar does not exist, bail out
            If Not (ValidObject(.vPos_scrollObj)) Then Exit Sub

            ' if this method has been turned off, then bail out
            If gMethod_vPos_scrollObj_Enabled = False Then Exit Sub

            ' +1 if moved up 1 full bar, -1 if moved down 1 full bar
            deltaScrollValue_Normalized = (.vPos_scrollObj.Value - .vPos_scroll_lastValue) / _
                ((.vPos_scrollObj.Maximum - .vPos_scrollObj.LargeChange + 1) - .vPos_scrollObj.Minimum)

            ' if there was no change, then bail out
            If deltaScrollValue_Normalized = 0 Then Exit Sub

            ' remember the new value of the scrollbar
            .vPos_scroll_lastValue = .vPos_scrollObj.Value

            ' scroll bar adds +4 to -4 times DisplayHeight

            ' convert to a -4 to +4 value
            deltaScrollValue_Normalized = deltaScrollValue_Normalized * Scroll_Range

            ' update all displays in this box
            For dNum = 0 To (.numDisp - 1)

                With gBox(bNum).disp(dNum)

                    ' calc the amount of change
                    delta_yEU = .dispHeight_yEu * deltaScrollValue_Normalized

                    ' Update vert scale given new yEU Top/Bot values.
                    Call vScale_SetWithNew_yEU_values_OneDisp(bNum, dNum, _
                        (.dispTop_yEu + delta_yEU), (.dispBot_yEu + delta_yEU), gEndUser_GRAPHICS)

                End With

            Next dNum

            ' Clear 1 DispBox and then Redraw it
            Call Redraw_OneBox(bNum, gEndUser_GRAPHICS)

        End With

    End Sub

    ' -------------------------------------------------------------------------
    ' FUNCTION    Box_Clear_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Clear DispBox
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Box_Clear_OneBox(ByVal bNum As Short, ByRef g_form As System.Drawing.Graphics)

        ' if we are drawing to the PRINTER, then get out of here
        If DrawingTo_PRINTER(g_form) Then Exit Sub

        ' turn on AutoRedraw
        Call Set_AutoRedraw_Property(True, g_form)

        ' Clear the Display box
        With gBox(bNum)

            Clear_Rectangle(.dispBoxLeft_xCoord, .dispBoxTop_yCoord, _
                .dispBoxRight_xCoord, .dispBoxBot_yCoord, g_form)

        End With

        ' turn off AutoRedraw
        Call Set_AutoRedraw_Property(False, g_form)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Set_AutoRedraw_Property()
    '
    ' PURPOSE     Sets the AutoDraw property (which enables automatically 
    '             drawing when window is reactivated).
    '
    '               THIS DOES NOT APPLY IN VISUAL BASIC >= 7
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Set_AutoRedraw_Property(ByVal newAutoDrawMode As Short, _
        ByRef g_form As System.Drawing.Graphics)

        ' if we are drawing to the PRINTER, then get out of here
        If DrawingTo_PRINTER(g_form) Then Exit Sub

        ' set the auto draw mode
        '
        '     OLD: g_form.AutoRedraw = newAutoDrawMode

        ' THERE IS NO AUTO REDRAW IN VB 7; therefore, we do nothing here.
        ' THERE IS NO AUTO REDRAW IN VB 7; therefore, we do nothing here.
        ' THERE IS NO AUTO REDRAW IN VB 7; therefore, we do nothing here.

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Plot_Clear_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Clear plot region for this display box
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Plot_Clear_OneBox(ByVal bNum As Short, ByRef g_form As System.Drawing.Graphics)

        Dim dNum As Short
        For dNum = 0 To (gBox(bNum).numDisp - 1)

            Call Plot_Clear_OneDisp(bNum, dNum, g_form)

        Next dNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Plot_Clear_OneDisp()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Clear plot region for this display
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Plot_Clear_OneDisp(ByVal bNum As Short, ByVal dNum As Short, _
        ByRef g_form As System.Drawing.Graphics)

        Call Set_AutoRedraw_Property(True, g_form)

        With gBox(bNum).disp(dNum)

            ' Fill the plot region w/ white.
            Clear_Rectangle( _
                (.dispLeft_xCoord + 1), _
                (.dispTop_yCoord + 1), _
                (gBox(bNum).vLabelBoxLeft_xCoord), _
                (.dispBot_yCoord - 1), g_form)

        End With

        Call Set_AutoRedraw_Property(False, g_form)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Marker_Draw_Line_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Draw the marker in Display.  If 'drawNewMarker' is True, we draw
    '             the marker at the new position; otherwise, we clear the old marker.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Marker_Draw_Line_OneBox(ByVal bNum As Short, ByVal drawNewMarker As Short, _
        ByRef thisBox As DispBoxParm, ByRef g_form As System.Drawing.Graphics)

        Dim point_xCoord, dispTop_yCoord, dispBot_yCoord As Integer
        Dim dNum, mkrNum As Short

        ' don't do anything in here if there are no displays in this DisplayBox.
        If (thisBox.numDisp <= 0) Then Exit Sub

        ' don't do anything in here if this DispBox does not support markers
        If (thisBox.supportMarkers = False) Then Exit Sub

        With thisBox

            Call Set_AutoRedraw_Property(True, g_form)

            ' ---------------------------------------------------
            ' Clear previous markers.

            ' if there is a previous marker to clear...
            If (drawNewMarker = False) Then

                Set_gEndUser_PEN(kWhite, g_form) ' Set pen to WHITE (i.e. erase)

                ' scan thru the markers in the display
                For mkrNum = 0 To (.numMkrs - 1)

                    ' with display that held old marker...
                    dNum = .prevMarkerPos_dNum(mkrNum)

                    If (dNum <> NO_PREVIOUS_MKR) Then

                        ' with Display that held previous marker...
                        With thisBox.disp(dNum)

                            ' erase old marker line
                            dispTop_yCoord = .dispTop_yCoord + 1
                            dispBot_yCoord = .dispBot_yCoord

                            ' calc position of previous marker
                            point_xCoord = Translate_xEU_to_xCoord(bNum, dNum, _
                                thisBox.prevMarkerPos_xEU(mkrNum))

                            ' make sure we do not overflow
                            If (point_xCoord >= thisBox.vLabelBoxLeft_xCoord) Then
                                point_xCoord = (thisBox.vLabelBoxLeft_xCoord - 1)
                            End If

                            ' draw red vertical line
                            g_form.DrawLine(gEndUser_PEN, point_xCoord, _
                                dispTop_yCoord, point_xCoord, dispBot_yCoord)

                        End With
                    End If
                Next mkrNum
            End If

            ' ---------------------------------------------------
            ' Draw new marker.

            ' if we are being asked to draw the new marker...
            If drawNewMarker = True Then

                Set_gEndUser_PEN(kRed, g_form) ' Set pen to RED

                ' scan thru the markers in the display
                For mkrNum = 0 To (.numMkrs - 1)

                    ' with Display that holds new marker...
                    dNum = .markerPos_dNum(mkrNum)

                    With thisBox.disp(dNum)

                        ' erase old marker line
                        dispTop_yCoord = .dispTop_yCoord + 1
                        dispBot_yCoord = .dispBot_yCoord

                        ' calc position of previous marker
                        point_xCoord = Translate_xEU_to_xCoord(bNum, dNum, thisBox.markerPos_xEU(mkrNum))

                        ' make sure we do not overflow
                        If (point_xCoord >= thisBox.vLabelBoxLeft_xCoord) Then
                            point_xCoord = (thisBox.vLabelBoxLeft_xCoord - 1)
                        End If

                        ' draw vertical red line
                        g_form.DrawLine(gEndUser_PEN, point_xCoord, _
                            dispTop_yCoord, point_xCoord, dispBot_yCoord)

                    End With

                    ' there is no longer a previous a marker that needs being cleared
                    .prevMarkerPos_dNum(mkrNum) = NO_PREVIOUS_MKR

                Next mkrNum
            End If

            Call Set_AutoRedraw_Property(False, g_form)

            Set_gEndUser_PEN(kBlack, g_form)                ' Set pen to BLACK

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_Form_MouseDown()
    '
    ' PURPOSE     Invoked by clicking on in the end user window (i.e. a VB form).
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_Form_MouseDown(ByVal eventSender As System.Object, _
        ByVal eventArgs As System.Windows.Forms.MouseEventArgs)

        ' -------------------------------------------------------------------
        ' Learn about the Mouse Down event

        ' if there are no displayBoxes, or the window is hidden, then exit
        If DisplayBoxes_Are_Not_Visible(gEndUser_GRAPHICS) Then Exit Sub

        ' true if mouse button is down
        Dim Button As Short = eventArgs.Button \ &H100000

        ' true if shift key is down
        Dim Shift As Short = System.Windows.Forms.Control.ModifierKeys \ &H10000

        ' calculate location of mouse down within the form
        ' OLD: Dim x_twip As Single = VB6.PixelsToTwipsX(eventArgs.X)
        ' OLD: Dim y_twip As Single = VB6.PixelsToTwipsY(eventArgs.Y)
        Dim x As Integer = eventArgs.X
        Dim y As Integer = eventArgs.Y

        ' determine if this occured within a display box, and if so, which one
        Dim bNum, mkrNum, mouseIsInADisplayBox As Short
        mouseIsInADisplayBox = False
        For bNum = 0 To gNumBoxes - 1 ' scan through all DisplayBoxes

            With gBox(bNum)

                If (x > .dispBoxLeft_xCoord) And (x < .dispBoxRight_xCoord) And _
                   (y < .hLowerLabel_Top_yCoord) And (y > .hUpperLabel_Bot_yCoord) Then

                    mouseIsInADisplayBox = True
                    GoTo SearchForDisplayBoxEnd

                End If

            End With

        Next bNum

SearchForDisplayBoxEnd:

        ' -------------------------------------------------------------------
        ' If this mouse down was not in a Display Box, then do nothing.

        If Not mouseIsInADisplayBox Then Exit Sub

        ' -------------------------------------------------------------------
        ' Calculate new and old marker positions.

        With gBox(bNum)

            ' don't do anything in here if there are no displays in this DisplayBox.
            If (.numDisp <= 0) Then Exit Sub

            ' don't do anything in here if this DispBox does not support markers
            If (.supportMarkers = False) Then Exit Sub

            ' remember old marker position, so we can clear it later
            For mkrNum = 0 To .numMkrs

                .prevMarkerPos_dNum(mkrNum) = .markerPos_dNum(mkrNum)
                .prevMarkerPos_xEU(mkrNum) = .markerPos_xEU(mkrNum)

                ' determine which disp in the DispBox received the mouse down.
                .markerPos_dNum(mkrNum) = Int(((y - .hUpperLabel_Bot_yCoord) / _
                    (.hLowerLabel_Top_yCoord - .hUpperLabel_Bot_yCoord)) * .numDisp)

                If (.markerPos_dNum(mkrNum) < 0) Then .markerPos_dNum(mkrNum) = 0
                If (.markerPos_dNum(mkrNum) >= .numDisp) Then .markerPos_dNum(mkrNum) = (.numDisp - 1)

                ' calc the point # of the mouse down in the Display
                .markerPos_xEU(mkrNum) = Translate_xCoord_to_xEU(bNum, .markerPos_dNum(mkrNum), x)

            Next mkrNum

        End With

        ' -------------------------------------------------------------------
        ' Erase the old marker

        Call Marker_Draw_Line_OneBox(bNum, False, gBox(bNum), gEndUser_GRAPHICS)

        ' -------------------------------------------------------------------
        ' Redraw grid and waveform plot, in case we nibbled at it (yet don't
        ' clear the entire display box).

        Call Marker_Draw_Line_OneBox(bNum, False, gBox(bNum), gEndUser_GRAPHICS)
        Dim dNum As Short
        For mkrNum = 0 To gBox(bNum).numMkrs

            dNum = gBox(bNum).prevMarkerPos_dNum(mkrNum)

            ' redraw the grid (in case we nibbled on it when we erased the maker)
            Call Grid_Draw_OneDisp(bNum, dNum, gBox(bNum), _
                gBox(bNum).disp(dNum), gEndUser_GRAPHICS)

            ' plot the full waveform (in case we nibbled on it when we drew the grid)
            Call Plot_FullScreen_OneDisp(bNum, dNum, gBox(bNum), _
                gBox(bNum).disp(dNum), gEndUser_GRAPHICS)

        Next mkrNum

        ' -------------------------------------------------------------------
        ' Draw the new marker in it's new position, in Red

        Call Marker_Draw_Line_OneBox(bNum, True, gBox(bNum), gEndUser_GRAPHICS)

        ' -------------------------------------------------------------------
        ' Plot the wave ONLY where the pervious marker was

        For dNum = 0 To (gBox(bNum).numDisp - 1)

            Call Plot_LastMarkerPos_OneDisp(bNum, dNum, gBox(bNum), _
                gBox(bNum).disp(dNum), gEndUser_GRAPHICS)

        Next dNum

        ' -------------------------------------------------------------------
        ' Update text at top of marker

        Call Marker_Draw_Text_OneBox(bNum, gBox(bNum), gEndUser_GRAPHICS)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Translate_xCoord_to_pointNum()
    '
    ' PURPOSE     Translates an xCoord value to a pointNum value for the
    '             {bNum, dispNum} Display.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function Translate_xCoord_to_pointNum(ByVal bNum As Short, _
        ByVal dNum As Short, ByVal xCoord As Short) As Integer

        With gBox(bNum).disp(dNum)

            Translate_xCoord_to_pointNum = _
                Int((xCoord - .dispLeft_xCoord) / .xCoord_Per_ptNum) + .dispLeft_ptNum

        End With

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    Translate_xEU_to_xCoord()
    '
    ' PURPOSE     Translates an xEU value to an xCoord value for the
    '             {bNum, dispNum} Display.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function Translate_xEU_to_xCoord(ByVal bNum As Short, ByVal dNum As Short, _
        ByVal xEU As Double) As Integer

        With gBox(bNum).disp(dNum)

            Translate_xEU_to_xCoord = Int((xEU - .dispLeft_xEU) * .xCoord_Per_xEU) + .dispLeft_xCoord

        End With

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    Translate_xEU_to_ptNum()
    '
    ' PURPOSE     Translates an xEU value to an pointNum value for the
    '             {bNum, dispNum} Display.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function Translate_xEU_to_ptNum(ByVal bNum As Short, ByVal dNum As Short, _
        ByVal xEU As Double) As Integer

        With gBox(bNum).disp(dNum)

            Translate_xEU_to_ptNum = Int((xEU - .dispLeft_xEU) * .xCoord_Per_xEU / _
                .xCoord_Per_ptNum) + .dispLeft_ptNum

        End With

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    Translate_xCoord_to_xEU()
    '
    ' PURPOSE     Translates an xCoord value to a pointNum value for the
    '             {bNum, dispNum} Display.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function Translate_xCoord_to_xEU(ByVal bNum As Short, ByVal dNum As Short, _
        ByVal xCoord As Short) As Double

        Dim x As Double

        With gBox(bNum).disp(dNum)

            x = .xCoord_Per_xEU

            Translate_xCoord_to_xEU = ((xCoord - .dispLeft_xCoord) / _
                .xCoord_Per_xEU) + .dispLeft_xEU

        End With

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    Set_gEndUser_PEN()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             We set gEndUser_PEN to one of the following styles via the 'color' arg:
    '
    '             kBlack      SCREEN: solid black   PRINTER: solid black
    '             kGray       SCREEN: solid gray    PRINTER: mesh
    '             kBlue       SCREEN: solid blue    PRINTER: solid black
    '             kRed        SCREEN: solid red     PRINTER: solid black
    '             kWhite      SCREEN: solid white   PRINTER: solid white
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Set_gEndUser_PEN(ByVal color As Short, ByRef g_form As System.Drawing.Graphics)


        ' If we wanted to do a better job here, we could
        ' use Pen.Brush instead (which overrides .color)
        ' and tell GRAY to be draw to the printer (e.g. b/w)
        ' in a MESH style (instead of Solid Line). gsw 6/1/02


        ' if we are drawing to the PRINTER...
        If (DrawingTo_PRINTER(g_form)) Then

            If (color = kGray) Then
                gEndUser_PEN.Color = System.Drawing.Color.Gray
            ElseIf (color = kWhite) Then
                gEndUser_PEN.Color = System.Drawing.Color.White
            Else
                gEndUser_PEN.Color = System.Drawing.Color.Black
            End If

        End If

        ' else, we are drawing to the SCREEN (i.e. crt monitor)
        If Not (DrawingTo_PRINTER(g_form)) Then

            If (color = kGray) Then
                gEndUser_PEN.Color = System.Drawing.Color.Gray
            ElseIf (color = kWhite) Then
                gEndUser_PEN.Color = System.Drawing.Color.White
            ElseIf (color = kRed) Then
                gEndUser_PEN.Color = System.Drawing.Color.Red
            ElseIf (color = kBlue) Then
                gEndUser_PEN.Color = System.Drawing.Color.Blue
            ElseIf (color = kBlack) Then
                gEndUser_PEN.Color = System.Drawing.Color.Black
            End If

        End If

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Box_Clear_AllBoxes()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Clear all DispBoxes
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Box_Clear_AllBoxes(ByRef g_form As System.Drawing.Graphics)
        Dim bNum As Short
        For bNum = 0 To gNumBoxes - 1
            Call Box_Clear_OneBox(bNum, g_form)
        Next bNum
    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Plot_Clear_AllBoxes()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Clear all the plot regions for all the DispBoxes
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Plot_Clear_AllBoxes(ByRef g_form As System.Drawing.Graphics)
        Dim bNum As Short
        For bNum = 0 To gNumBoxes - 1
            Plot_Clear_OneBox(bNum, g_form)
        Next bNum
    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_hPos_decrement_Click()
    '
    ' PURPOSE     Decrement horizontal scale (e.g. ms/div).
    '             Invoked by a CLICK on the hPos_decrement (horiz scale) button.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_hPos_decrement_Click(ByVal bNum As Short)

        ' don't do anything in here if there are not displays in this DisplayBox.
        If (gBox(bNum).numDisp <= 0) Then Exit Sub

        ' decrement horiz scale 50%
        Call hScale_StretchScale_OneBox(bNum, 0.5, gEndUser_GRAPHICS)

        ' redraw 1 DispBox
        Call Redraw_OneBox(bNum, gEndUser_GRAPHICS)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_hPos_increment_Click()
    '
    ' PURPOSE     Increment horizontal scale (e.g. ms/div).
    '             Invoked by a CLICK on the hPos_increment (horiz scale) button.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_hPos_increment_Click(ByVal bNum As Short)

        ' don't do anything in here if there are not displays in this DisplayBox.
        If (gBox(bNum).numDisp <= 0) Then Exit Sub

        ' increment horiz scale 200%
        Call hScale_StretchScale_OneBox(bNum, 2.0#, gEndUser_GRAPHICS)

        Call Redraw_OneBox(bNum, gEndUser_GRAPHICS) ' redraw 1 DispBox

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_ChanOpt_Button_Click()
    '
    ' PURPOSE     Invoked by clicking on the Channel Options button.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_ChanOpt_Button_Click()

        ' Open iNet World's Network Page.
        Open_instruNet_Window_VB(iNetCM_OpenWindow_Network)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_hPos_scroll_KeyDown()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Update horizontal position for this DispBox.  Invoked by key down
    '             in Horiz scrollbar.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_hPos_scroll_KeyDown(ByRef Index As Short, ByRef KeyCode As Short, _
        ByRef Shift As Short)

        Dim bNum As Short
        bNum = Index

        ' don't do anything in here if there are not displays in this DisplayBox.
        If (gBox(bNum).numDisp <= 0) Then Exit Sub

        ' if the scrollbar does not exist, bail out
        If Not (ValidObject(gBox(bNum).hPos_scrollObj)) Then Exit Sub

        ' reset to left edge
        If KeyCode = &H30S Then

            ' reset the left edge to 0
            gBox(bNum).hPos_scrollObj.Value = gBox(bNum).hPos_scrollObj.Minimum

            ' update the hScroll bar
            METHOD_hPos_scroll_Change(bNum)

        End If

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_hPos_scroll_Change()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Update horizontal position for this DispBox.
    '             Invoked by key down in Horiz scrollbar.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_hPos_scroll_Change(ByRef Index As Short)

        Dim left_xEU_NEW, delta_xEU, ptsPerXunit, firstPt_Xunits As Double
        Dim dispLeft_ptNum_NEW, dispLeft_ptNum_original, numValidPts As Integer
        Dim scrollValue_Normalized, deltaScrollValue_Normalized As Double
        Dim bNum, dNum As Short

        bNum = Index

        ' don't do anything in here if there are not displays in this DisplayBox.
        If (gBox(bNum).numDisp <= 0) Then Exit Sub

        ' get the # of data pts and the data's sample rate (for TOP display)
        Call GetDataParams(bNum, TOP_DISPLAY_NUM, ptsPerXunit, numValidPts, firstPt_Xunits)

        ' if we do not have any points in the display, then don't allow scrolling
        If (numValidPts = 0) Then Exit Sub

        ' if this method has been turned off, then bail out
        If gMethod_hPos_scrollObj_Enabled = False Then Exit Sub

        ' if the scrollbar does not exist, bail out
        If Not (ValidObject(gBox(bNum).hPos_scrollObj)) Then Exit Sub

        ' w/ the TOP-MOST display...
        Const Scroll_Range As Short = 4
        With gBox(bNum).disp(TOP_DISPLAY_NUM)

            ' -----------------------------------
            ' XY Display

            If (.displayType = kXYDisplay) Then

                ' +1 if moved up 1 full bar, -1 if moved down 1 full bar
                deltaScrollValue_Normalized = _
                    (gBox(bNum).hPos_scrollObj.Value - gBox(bNum).hPos_scroll_lastValue) / _
                    ((gBox(bNum).hPos_scrollObj.Maximum - gBox(bNum).hPos_scrollObj.LargeChange + 1) - _
                    gBox(bNum).hPos_scrollObj.Minimum)

                ' if there was no change, then bail out
                If deltaScrollValue_Normalized = 0 Then Exit Sub

                ' remember the new value of the scrollbar
                gBox(bNum).hPos_scroll_lastValue = gBox(bNum).hPos_scrollObj.Value

                ' scroll bar adds +4 to -4 times DisplayWidth

                ' convert to a -4 to +4 value
                deltaScrollValue_Normalized = deltaScrollValue_Normalized * Scroll_Range

                ' update all displays in this box
                For dNum = 0 To (gBox(bNum).numDisp - 1)

                    With gBox(bNum).disp(dNum)

                        ' calc the amount of change
                        delta_xEU = .dispWidth_xEU * deltaScrollValue_Normalized

                        ' Update vert scale given new yEU Top/Bot values.
                        Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, _
                            (.dispLeft_xEU + delta_xEU), (.dispRight_xEU + delta_xEU), gEndUser_GRAPHICS)

                    End With

                Next dNum

                ' Clear 1 DispBox and then Redraw it
                Call Redraw_OneBox(bNum, gEndUser_GRAPHICS)

                ' -----------------------------------
                ' Y vs INDEX Display

            Else

                ' remember the old left edge value
                dispLeft_ptNum_original = .dispLeft_ptNum

                ' 0 is all the way to the left, 1 all the way to right
                scrollValue_Normalized = (gBox(bNum).hPos_scrollObj.Value - gBox(bNum).hPos_scrollObj.Minimum) _
                    / ((gBox(bNum).hPos_scrollObj.Maximum - gBox(bNum).hPos_scrollObj.LargeChange + 1) _
                    - gBox(bNum).hPos_scrollObj.Minimum)

                ' remember the new value of the scrollbar
                gBox(bNum).hPos_scroll_lastValue = gBox(bNum).hPos_scrollObj.Value

                ' calc new left edge ptNum, based on current scroll bar value
                dispLeft_ptNum_NEW = scrollValue_Normalized * numValidPts

                ' don't allows the scrolling to go past the waveform end points
                If dispLeft_ptNum_NEW < 0 Then dispLeft_ptNum_NEW = 0
                If dispLeft_ptNum_NEW > numValidPts Then dispLeft_ptNum_NEW = numValidPts - 1

                ' if the left edge changed...
                If (dispLeft_ptNum_original <> dispLeft_ptNum_NEW) Then

                    ' calc the new left edge xEu value
                    left_xEU_NEW = dispLeft_ptNum_NEW / ptsPerXunit

                    ' update all displays in this box
                    For dNum = 0 To (gBox(bNum).numDisp - 1)

                        ' Update horiz scale (pos/scale) given new xEU Left/Right values.
                        Call hScale_SetWithNew_xEU_values_OneDisp(bNum, dNum, left_xEU_NEW, _
                            (left_xEU_NEW + (.dispRight_xEU - .dispLeft_xEU)), gEndUser_GRAPHICS)

                    Next dNum

                    ' Clear 1 DispBox and then Redraw it
                    Call Redraw_OneBox(bNum, gEndUser_GRAPHICS)

                End If

            End If

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Redraw_AllBoxes()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Clear all DispBoxes and then redraw them.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Redraw_AllBoxes(ByRef g_form As System.Drawing.Graphics)

        Dim bNum As Short

        ' if there are no displayBoxes, or the window is hidden, then exit
        If DisplayBoxes_Are_Not_Visible(g_form) Then Exit Sub

        For bNum = 0 To gNumBoxes - 1

            ' redraw this box
            Call Redraw_OneBox(bNum, g_form)

        Next bNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Redraw_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Clear 1 DispBox and then Redraw it
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Redraw_OneBox(ByVal bNum As Short, ByRef g_form As System.Drawing.Graphics)

        ' if there are no displayBoxes, or the window is hidden, then exit
        If DisplayBoxes_Are_Not_Visible(g_form) Then Exit Sub

        ' clear entire DisplayBox
        Call Box_Clear_OneBox(bNum, g_form)

        ' draw the grid (we draw stuff over it)
        Call Grid_Draw_OneBox(bNum, g_form)

        ' draw lower hScale label
        Call hScale_DrawLowerLabel_OneBox(bNum, gBox(bNum), g_form)

        ' draw rightside vScale label
        Call vScale_DrawLabel_OneBox(bNum, g_form)

        ' draw vertical marker line
        Call Marker_Draw_Line_OneBox(bNum, True, gBox(bNum), g_form)

        ' draw marker text at top of boxes
        Call Marker_Draw_Text_OneBox(bNum, gBox(bNum), g_form)

        ' plot the data
        Call Plot_FullScreen_OneBox(bNum, g_form)

        ' draw the borders around the display
        Call Borders_Draw_OneBox(bNum, g_form)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    hScale_SetWithNew_xEU_values_OneDisp()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Given a new dispLeft_xEU/dispRight_xEU, we update the horiz scale params.
    '
    ' PARAMS        INPUT    dispLeft_xEU/dispRight_xEU, disp(cNum).dataArrayY.ptsPerXunit
    '
    '               OUTPUT  .xCoord_Per_xEU, xCoord_Per_ptNum
    '                       .dispLeft_ptNum, .dispRight_ptNum, .dispWidth_numPts
    '
    '             This also updates the lower hScale label text if this is the TOP-MOST display.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub hScale_SetWithNew_xEU_values_OneDisp(ByVal bNum As Short, ByVal dNum As Short, _
        ByVal dispLeft_xEU As Double, ByVal dispRight_xEU As Double, _
        ByRef g_form As System.Drawing.Graphics)

        Dim ptsPerXunit, firstPt_Xunits As Double
        Dim numValidPts As Integer

        ' don't do anything in here if there are no displays in this DisplayBox.
        If (gBox(bNum).numDisp <= dNum) Then Exit Sub

        ' get the # of data pts and the data's sample rate
        Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)

        With gBox(bNum).disp(dNum)

            ' bail out if these values are not new
            If .dispLeft_xEU = dispLeft_xEU And .dispRight_xEU = dispRight_xEU Then Exit Sub

            .dispLeft_xEU = dispLeft_xEU ' horiz Left/Right values in xEU coords
            .dispRight_xEU = dispRight_xEU
            .dispWidth_xEU = dispRight_xEU - dispLeft_xEU

            ' xCoords per xEU (horiz plot scale)
            .xCoord_Per_xEU = (.dispRight_xCoord - .dispLeft_xCoord) / (.dispRight_xEU - .dispLeft_xEU)

            ' help code recover from no points scenario
            If (.xCoord_Per_xEU = 0) Then .xCoord_Per_xEU = 1

            ' make sure sample rate is not 0, to avoid errors
            If (ptsPerXunit = 0) Then EndProgram_iNet(("why sRate 0?"))

            ' set the plotted pts-per-xUnit (e.g. pts-per-sec)
            Call hScale_SetPtsPerHscale_OneDisp(bNum, dNum, ptsPerXunit)

            ' Debugging
            ' temp = .dispWidth_numPts: tempB = .dispLeft_ptNum: tempC = .dispRight_ptNum
            ' tempD = .xCoord_Per_ptNum: tempE = .dispRight_xCoord: tempF = .dispLeft_xCoord

            ' If this is the top most display, then update the Lower hScale label
            If (dNum = TOP_DISPLAY_NUM) Then

                ' Draw Lower hScale label at base of DisplayBox.
                Call hScale_DrawLowerLabel_OneBox(bNum, gBox(bNum), g_form)

            End If

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    hScale_SetPtsPerHscale_OneDisp()
    '
    ' PURPOSE     Set the pts-per-hScale (based on the sample rate of the data).
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub hScale_SetPtsPerHscale_OneDisp(ByVal bNum As Short, ByVal dNum As Short, _
        ByVal ptsPerXunit_thisDisp As Double)

        If ptsPerXunit_thisDisp <= 1.0E-30 Then EndProgram_iNet(("bad sRate"))

        With gBox(bNum).disp(dNum)

            ' waveform ptNum corresponding to left/right/width
            ' (assumes waveform data exists, which it might not)
            .dispLeft_ptNum = .dispLeft_xEU * ptsPerXunit_thisDisp
            .dispRight_ptNum = .dispRight_xEU * ptsPerXunit_thisDisp
            .dispWidth_numPts = 1 + .dispRight_ptNum - .dispLeft_ptNum

            ' help code recover from no points scenario
            If (.dispWidth_numPts <= 0) Then .dispWidth_numPts = 1

            ' xCoords per pointNum (horiz plot scale)
            .xCoord_Per_ptNum = (.dispRight_xCoord - .dispLeft_xCoord) / .dispWidth_numPts

            ' help code recover from no points scenario
            If (.xCoord_Per_ptNum = 0) Then .xCoord_Per_ptNum = 1

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Marker_Draw_Text_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Updates marker information that is shown at the top of the display
    '             (e.g. "Time Waves       Ch1 = {.5, 2.6} ").
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Marker_Draw_Text_OneBox(ByVal bNum As Short, ByRef thisBox As DispBoxParm, _
        ByRef g_form As System.Drawing.Graphics)

        Dim data As Single
        Dim mkrNum, dNum As Short
        Dim hUpperLabel_text As String
        Dim lastValid_ptNum, point_ptNum, numValidPts, Pen_yCoord, Pen_xCoord As Integer
        Dim ptsPerXunit, firstPt_Xunits As Double

        ' don't do anything in here if there are no displays in this DisplayBox.
        If (thisBox.numDisp <= 0) Then Exit Sub

        ' get the display Title text
        hUpperLabel_text = thisBox.hDispBoxTitle_text

        ' scan thru the markers in the display
        For mkrNum = 0 To (thisBox.numMkrs - 1)

            ' get Display that contains the marker
            dNum = thisBox.markerPos_dNum(mkrNum)

            With thisBox.disp(dNum)

                ' get ptNum at marker position
                point_ptNum = Translate_xEU_to_ptNum(bNum, dNum, thisBox.markerPos_xEU(mkrNum))

                ' get the # of data pts and the data's sample rate
                Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)

                ' in data buffer, last valid ptNum
                lastValid_ptNum = numValidPts - 1

                ' make sure ptNum does not overflow the wave
                If (point_ptNum < 0) Then point_ptNum = 0
                If (point_ptNum > lastValid_ptNum) Then point_ptNum = lastValid_ptNum

                ' get local_data(), or gDigiAddr()
                If point_ptNum <= lastValid_ptNum And point_ptNum >= 0 Then
                    If (.cNum = NO_DIGI_CHAN) Then
                        data = thisBox.disp(dNum).dataArrayY.data(point_ptNum)
                    Else
                        data = gDigiChan(.cNum).dataArrayY.data(point_ptNum)
                    End If
                Else
                    data = 0
                End If

                ' get yEU value and round off till the last 6 digits
                .valueAtMarker_yEU(mkrNum) = data

                ' if this box supports markers...
                If (thisBox.supportMarkers = True) Then

                    ' add marker name (e.g. "Time Waves       Ch1 = {.5, 2.6}")
                    hUpperLabel_text = hUpperLabel_text & _
                        "         " & _
                        .name & _
                        " = {" & _
                        NumToStr(thisBox.markerPos_xEU(mkrNum)) & _
                        ", " & _
                        NumToStr(.valueAtMarker_yEU(mkrNum)) _
                        & "}"

                End If

            End With

        Next mkrNum

        ' ------------------------------------------------
        ' Draw upper Label text.

        Call Set_AutoRedraw_Property(True, g_form)

        ' fill the entire UpperLabel region w/ white
        Clear_Rectangle((thisBox.dispBoxLeft_xCoord + 1), (thisBox.dispBoxTop_yCoord + 1), _
            (thisBox.dispBoxRight_xCoord - 1), (thisBox.hUpperLabel_Bot_yCoord - 1), g_form)

        ' Sets the drawing style to a black
        Call Set_gEndUser_PEN(kBlack, g_form)

        ' move pen to upper label region
        Pen_xCoord = thisBox.dispBoxLeft_xCoord + CrtPix_to_GraphicsPix(5, g_form)
        Pen_yCoord = thisBox.dispBoxTop_yCoord + vSPACE_yPix(g_form)

        ' print the marker text
        g_form.DrawString(hUpperLabel_text, gEndUser_FONT, Brushes.Black, Pen_xCoord, Pen_yCoord)

        Call Set_AutoRedraw_Property(False, g_form)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    GetDataParams()
    '
    ' PURPOSE     Loads args 'sRate' and 'numValidPts' with the sample rate and the
    '             the # of pts for the data tied to the {bNum, dNum display}.
    '             This data is either stored linked to the Digitize structure, or the
    '             Display structure.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub GetDataParams(ByVal bNum As Short, ByVal dNum As Short, _
        ByRef ptsPerXunit As Double, ByRef numValidPts As Integer, _
        ByRef firstPt_Xunits As Double)

        With gBox(bNum).disp(dNum)

            ' debugging
            ' a = gBox(bNum).disp(dNum).cNum
            ' b = gBox(bNum).disp(dNum).dataArrayY.numValidPts
            ' c = gBox(bNum).disp(dNum).dataArrayY.ptsPerXunit

            ' if the Display uses local data...
            If (.cNum = NO_DIGI_CHAN) Then

                ' then get the data from the display
                numValidPts = .dataArrayY.numValidPts
                ptsPerXunit = .dataArrayY.ptsPerXunit
                firstPt_Xunits = .dataArrayY.firstPt_Xunits

                ' else, get params associated w/ digitize data
            Else

                numValidPts = gDigiChan(.cNum).dataArrayY.numValidPts
                ptsPerXunit = gDigiChan(.cNum).dataArrayY.ptsPerXunit
                firstPt_Xunits = gDigiChan(.cNum).dataArrayY.firstPt_Xunits

            End If

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Reset_Controls_AllBoxes()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Initialize horiz/vert scroll/scale to default values.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Sub Reset_Controls_AllBoxes(ByRef thingsToReset As Short, _
        ByRef g_form As System.Drawing.Graphics)

        Dim bNum As Short

        ' if there are no displayBoxes, or the window is hidden, then exit
        If DisplayBoxes_Are_Not_Visible(g_form) Then Exit Sub

        For bNum = 0 To gNumBoxes - 1

            Call Reset_Controls_OneBox(bNum, gBox(bNum), thingsToReset, g_form)

        Next bNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Plot_FullScreen_AllBoxes()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Plot data in all DispBoxes
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Plot_FullScreen_AllBoxes(ByRef g_form As System.Drawing.Graphics)

        ' draw red vertical line
        ' Playing:  g_form.DrawLine(gEndUser_PEN, 0, 0, 1500, 1500)

        Dim bNum As Short
        For bNum = 0 To gNumBoxes - 1

            Call Plot_FullScreen_OneBox(bNum, g_form)

        Next bNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Grid_Draw_AllBoxes()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Draw the grid in all DispBoxes
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Grid_Draw_AllBoxes(ByRef g_form As System.Drawing.Graphics)
        Dim bNum As Short
        For bNum = 0 To gNumBoxes - 1 ' scan through all DisplayBoxes
            Call Grid_Draw_OneBox(bNum, g_form)
        Next bNum
    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    vScale_DrawLabel_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Draw the right-side vScale labels in all DispBoxes
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub vScale_DrawLabel_OneBox(ByVal bNum As Short, ByRef g_form As System.Drawing.Graphics)
        Dim dNum As Short
        For dNum = 0 To (gBox(bNum).numDisp - 1)
            Call vScale_DrawLabel_OneDisp(bNum, dNum, gBox(bNum).disp(dNum), g_form)
        Next dNum
    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Borders_Draw_AllBoxes()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Draw the borders within in all DispBoxes
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Borders_Draw_AllBoxes(ByRef g_form As System.Drawing.Graphics)
        Dim bNum As Short
        For bNum = 0 To gNumBoxes - 1 ' scan through all DisplayBoxes
            Call Borders_Draw_OneBox(bNum, g_form)
        Next bNum
    End Sub

    ' -------------------------------------------------------------------------
    ' FUNCTION    Initialize_instruNet_Displays()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Initialize instruNet display code.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------

    ' true if we have initialized our Displays
    Private gWeHaveInitializedDisplays As Boolean = False

    Public Sub Initialize_instruNet_Displays( _
        ByRef endUser_Class As EndUser_Window, _
        ByRef endUser_form As System.Windows.Forms.Form)

        ' ------------------------------------------    
        ' Initialize the INET_Display module

        ' make sure we don't initialize twice
        If IsNothing(gWeHaveInitializedDisplays) Then
            gWeHaveInitializedDisplays = True
        ElseIf (gWeHaveInitializedDisplays = True) Then
            MsgBox("Only call Initialize_instruNet_MAIN() once per Application launch.")
            Exit Sub
        ElseIf (gWeHaveInitializedDisplays = False) Then
            gWeHaveInitializedDisplays = True
        End If

        ' main window created by end user (type = FORM)
        gEndUser_FORM = endUser_form

        ' create graphics device for FORM, gsw 6/1/02
        gEndUser_GRAPHICS = gEndUser_FORM.CreateGraphics

        ' initialize the drawing pen (black, 1 pixel wide)
        gEndUser_PEN = New Pen(System.Drawing.Color.Black, 1)

        ' initialize font for drawing
        gEndUser_FONT = New Font("Arial", 10)

        ' Init scroll bar methods.
        gMethod_vPos_scrollObj_Enabled = True
        gMethod_hPos_scrollObj_Enabled = True

        ' return string (carriage return + line feed)
        gReturnStr = Chr(13) & Chr(10)

        ' This CLASS created by the end user that contains a window 
        ' (i.e. Visual Basic FORM) and several callback functions:
        ' EndUser_SegmentProcessor(), EndUser_SaveLoadResults(), and
        ' EndUser_PostAcqAnalysis()
        gEndUser_CLASS = endUser_Class

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Close_instruNet_Displays()
    '
    ' PURPOSE     Closes the instruNet Display code.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Close_instruNet_Displays()

        ' Note: It is possible that the end user will have
        ' no displays, and this rountine will subsequently
        ' do very little.

        ' record that the displays are no longer initialized
        gWeHaveInitializedDisplays = False

        ' Delete the PEN obj, gsw 6/1/02
        If Not IsNothing(gEndUser_PEN) Then gEndUser_PEN.Dispose()

        ' Delete the FONT obj, gsw 6/1/02
        If Not IsNothing(gEndUser_FONT) Then gEndUser_FONT.Dispose()

        ' Delete the GRAPHICS device (used to draw on a FORM), gsw 6/1/02
        If Not IsNothing(gEndUser_GRAPHICS) Then gEndUser_GRAPHICS.Dispose()

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Marker_Draw_Line_AllBoxes()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Draw the marker in all DispBoxes
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Marker_Draw_Line_AllBoxes(ByRef g_form As System.Drawing.Graphics)
        Dim bNum As Short
        For bNum = 0 To gNumBoxes - 1       ' draw new marker
            Call Marker_Draw_Line_OneBox(bNum, True, gBox(bNum), g_form)
        Next bNum
    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Marker_Draw_Text_AllBoxes()
    '
    ' PURPOSE     This is a low level routine that is typically not used by
    '             the end user.
    '
    '             Updates marker information that is shown on the front panel
    '             as text, all DispBoxes
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Marker_Draw_Text_AllBoxes(ByRef g_form As System.Drawing.Graphics)
        Dim bNum As Short
        For bNum = 0 To gNumBoxes - 1
            Call Marker_Draw_Text_OneBox(bNum, gBox(bNum), g_form)
        Next bNum
    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_Print_Screen_Button()
    '
    ' PURPOSE     Print the main window (i.e. VB Form), one printed page for
    '             the entire window (i.e. Form) , independent of what is on it.
    '
    '             This routine is sometimes called when the PRINT button is clicked.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub METHOD_Print_Screen_Button()

        ' This is a helper class included in the VB project, within the
        ' INET_PRintForm.vb file, that prints a FORM (simple !). gsw 6/1/02
        Dim PrinterObj As New PrintForm()

        ' Print the FORM (i.e. window) on one page of paper
        PrinterObj.Print(gEndUser_FORM)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    METHOD_Print_All_DisplayBoxes_Button()
    '
    ' PURPOSE     Print to hard-copy printer all DisplayBoxes,one printed 
    '             page per displayBox.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------

    ' used to keep track of which display box is being printed
    Private gNextDisplayBoxToPrint_bNum As Short

    ' keep track of the pagesetup set up by end user
    Private printPageSettings As New PageSettings()

    ' True if we are printing for the 1st time
    Private printingForFirstTime As Boolean = True

    Public Sub METHOD_Print_All_DisplayBoxes_Button()

        Try

            ' tell it to print the 0th displayBox next
            gNextDisplayBoxToPrint_bNum = 0

            ' we create a new PrintDocument
            Dim pd As New PrintDocument()

            ' Add a Print Handler that is called by the OS when it wants to Print
            AddHandler pd.PrintPage, AddressOf Display_Utilities.Draw_OneDisplayBox_To_PRINTER

            ' Create a Graphics device for this printer
            '  OLD: Dim printer_Graphics As System.Drawing.Graphics = pd.PrinterSettings.CreateMeasurementGraphics

            ' Specifies a device pixel as the unit of measure (e.g. on a 1200 dpi
            ' printer, this is 1/1200th of an inch)
            '  OLD: printer_Graphics.PageUnit = System.Drawing.GraphicsUnit.Pixel

            ' if we are printing for the 1st time, then preset the Pagesetup
            If printingForFirstTime Then

                ' Set the page orientation to landscape.
                printPageSettings.Landscape = True

                ' Request half inch margins
                '  OLD: Dim margin_xPix As Integer = printer_Graphics.DpiX * 0.5
                '  OLD: Dim margin_yPix As Integer = printer_Graphics.DpiY * 0.5

                ' Create a new instance of Margins with 0.5" margins.
                '  OLD: Dim margins As New Margins(margin_xPix, margin_xPix, margin_yPix, margin_yPix)
                Dim margins As New Margins(50, 50, 50, 50)  ' 50/100th of an inch
                printPageSettings.Margins = Margins

                ' we only preset the page settings once per application launch
                printingForFirstTime = False

            End If

            ' Grab the stored page settings
            pd.DefaultPageSettings = printPageSettings

            ' Show the standard Print Dialog (e.g. Select printer)
            Dim PrintDialog1 As New PrintDialog()
            PrintDialog1.Document = pd
            Dim result As DialogResult = PrintDialog1.ShowDialog()

            ' If end user pressed OK, then print the document
            If (result = DialogResult.OK) Then

                ' Tell the OS to Print, and to send the print job to
                ' Draw_OneDisplayBox_To_PRINTER() multiple times, one page per DisplayBox
                pd.Print()

            End If

        Catch ex As Exception   ' Catch Printing Error

            MessageBox.Show("There is a printing problem.  ", ex.ToString())

        End Try

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Draw_OneDisplayBox_To_PRINTER()
    '
    ' PURPOSE     This is a low level routine that is typically not used by
    '             the end user.
    '
    '             Plot wave data on PRINTER for 1 display box. 
    '
    '             This is called by the Windows PRINTDOCUMENT
    '             class after being installed by
    '
    '             Use METHOD_Print_Screen_Button() if you
    '             want to print the entire window on one page.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Draw_OneDisplayBox_To_PRINTER( _
        ByVal sender As Object, ByVal ev As PrintPageEventArgs)

        ' --------------------------------------------------------------
        ' Local Variables

        Dim footer As String
        Dim bNum As Short

        ' if we have printed all the display boxes, bail out
        If (gNextDisplayBoxToPrint_bNum >= gNumBoxes) Then GoTo ExitPrintRoutine

        ' print this one display box on 1 page of paper.
        bNum = gNextDisplayBoxToPrint_bNum

        ' --------------------------------------------------------------
        ' Get a GRAPHICS device for this printer, and sets the x/y
        ' coordinate units to printer device pixels (e.g. on a 1200 dpi
        ' printer, this is 1/1200th of an inch)

        Dim printer_Graphics As System.Drawing.Graphics = ev.Graphics

        ' Specifies a device pixel as the unit of measure (e.g. on a 1200 dpi
        ' printer, this is 1/1200th of an inch)
        printer_Graphics.PageUnit = System.Drawing.GraphicsUnit.Pixel

        ' --------------------------------------------------------------
        ' Calc Footer string.

        ' Print the footer
        footer = VB6.Format(Today, "Long Date") & ", " & _
            VB6.Format(TimeOfDay, "hh:mm:ss AMPM") & ", Page #" & Str(bNum + 1)

        ' --------------------------------------------------------------
        ' Calc Footer and DisplayBox rectangles.

        ' There is a Tricky Issue Here:
        '
        '   The following MarginBound code is recommend by Microsoft and works 
        '   correctly with Adobe Distiller. However, on an HP Laser Jet 3200,
        '   the Laser acts like the coordinate system has 0,0 at the margin
        '   bound upper left, which means that the below code should set the
        '   x/y parameters to 0 instead of adding in the margin. Subsequently,
        '   on the HP LaserJet 3200, the image is shift down and to the right by
        '   .5" or so.

        ' calc rectangle to printer page, within the margins (Margin uses .01" unit)
        Dim pageRectF As New RectangleF( _
            ((ev.MarginBounds.Left + 2) * 0.01) * printer_Graphics.DpiX, _
            ((ev.MarginBounds.Top + 2) * 0.01) * printer_Graphics.DpiY, _
            ((ev.MarginBounds.Width - 4) * 0.01) * printer_Graphics.DpiX, _
            ((ev.MarginBounds.Height - 4) * 0.01) * printer_Graphics.DpiY)

        ' calc footer height, in printer pixels
        Dim footerHeight_yCoord As Integer = 1.5 * printer_Graphics.MeasureString(footer, gEndUser_FONT).Height

        ' calc rect that encloses the DisplayBox
        Dim displayBoxRectF As New RectangleF(pageRectF.Left, pageRectF.Top, _
            pageRectF.Width, pageRectF.Height - footerHeight_yCoord)

        ' --------------------------------------------------------------
        ' Calculate x/y coords that coorespond to a printed page, and
        ' update the DisplayBox and it's displays w/ these new coordinates.

        ' ReCalculate DisplayBox's x/y position coordinates, given an enclosing rectangle
        DisplayBox_Calculate_Given_EnclosingRect(bNum, displayBoxRectF, printer_Graphics)

        ' ReCalculate All Display x/y coords given the 1 enclosing DisplayBox
        DisplayBox_Calculate_Enclosed_Displays(bNum, printer_Graphics)

        ' --------------------------------------------------------------
        ' Print this 1 DisplayBox to 1 hard copy printed page

        Redraw_OneBox(bNum, printer_Graphics)

        ' --------------------------------------------------------------
        ' Print the footer

        ' draw the footer string
        printer_Graphics.DrawString(footer, gEndUser_FONT, Brushes.Black, _
            pageRectF.Left + CrtPix_to_GraphicsPix(2, printer_Graphics), _
            pageRectF.Bottom - (footerHeight_yCoord * 0.1))

        ' --------------------------------------------------------------
        ' ReCalculate x/y coords for this 1 displayBox, based on the screen.
        ' (sets global variables associated w/ this displayBox). We do this
        ' w.r.t. gEndUser_GRAPHICS which is the VB Form's Graphics device.

        DisplayBox_Calculate_DisplayBox_And_All_Displays(bNum, gEndUser_GRAPHICS)

        ' --------------------------------------------------------------
        ' Exit.

ExitPrintRoutine:

        ' next time, print the next display box
        gNextDisplayBoxToPrint_bNum += 1

        ' see if we have more pages to print
        ev.HasMorePages = IIf((gNextDisplayBoxToPrint_bNum < gNumBoxes), True, False)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    DrawingTo_PRINTER()
    '
    ' PURPOSE     Returns TRUE if drawing to printer; FALSE if drawing to screen.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function DrawingTo_PRINTER(ByRef g_form As System.Drawing.Graphics) As Boolean

        ' if the GRAPHICS device is the screen graphics device,
        ' then we are drawing to the SCREEN; otherwise, PRINTER
        DrawingTo_PRINTER = IIf(g_form.Equals(gEndUser_GRAPHICS), False, True)

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    Plot_Segment_OneDisp()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Plot the data in the display, from first_ptNum to last_ptNum.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Plot_Segment_OneDisp(ByVal bNum As Short, ByVal dNum As Short, _
        ByRef thisBox As DispBoxParm, ByRef thisDisp As DispParm, ByVal first_ptNum As Integer, _
        ByVal last_ptNum As Integer, ByRef g_form As System.Drawing.Graphics)

        ' -------------------------------------------------------------------
        ' Local Variables.

        Dim point_yCoord, point_xCoord, nextPoint_xCoord, nextPoint_yCoord, plotStep_ptNum As Integer
        Dim ptsPerXunit, firstPt_Xunits As Double
        Dim dataY, dataX As Single    ' iNet data is flt32
        Dim plotTheNextPoint, xy As Short
        Dim lastValid_ptNum, numValidPts, ptNum As Integer

        ' -------------------------------------------------------------------
        ' Initialize.

        ' true if doing XY display; false if doing Y vs index.
        xy = (thisDisp.displayType = kXYDisplay)

        ' Don't do anything in here if there are not displays in this DisplayBox.
        If (thisBox.numDisp <= 0) Then Exit Sub

        ' get the # of data pts and the data's sample rate (for the Y data)
        Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)

        ' in data buffer, last valid ptNum
        lastValid_ptNum = numValidPts - 1

        ' If there are no points to plot, then bail out
        If (first_ptNum > lastValid_ptNum) Then Exit Sub

        ' If there is no data to plot, then bail out.
        If (first_ptNum > last_ptNum) Then Exit Sub

        ' radius, in pixels, of little plotted dot
        Dim dotRadius_pixels As Integer = CrtPix_to_GraphicsPix(2, g_form)

        ' -------------------------------------------------------------------
        ' Do the plot.

        Call Set_AutoRedraw_Property(True, g_form)

        Set_gEndUser_PEN(kBlue, g_form)                ' Set pen to BLUE

        ' do the following w/ this Display
        With thisDisp

            ' if working with an Y vs index display...
            If (.displayType = kYindexDisplay) Then

                ' If the 1st point is to the left of the left edge, set it to the left edge...
                If (first_ptNum < .dispLeft_ptNum) Then first_ptNum = .dispLeft_ptNum

                ' If the last point is to the right of the right edge, set it to the right edge...
                If (last_ptNum > .dispRight_ptNum) Then last_ptNum = .dispRight_ptNum

            End If

            ' If the last plot point is past last valid data pt, then
            ' set it to the last valid data pt.
            If (last_ptNum > lastValid_ptNum) Then last_ptNum = lastValid_ptNum

            ' If we don't have any data to plot, then bail out...
            If (first_ptNum > last_ptNum) Then GoTo EndOfPlotting

            ' back up one, so that we can draw a line segment starting
            ' at the point before the plotted segment
            If (first_ptNum > 0) Then first_ptNum = first_ptNum - 1

            ' We don't plot the 1st point, since we need to prime the pump
            ' and locate the point before the 1st point, first.
            plotTheNextPoint = False

            ' Y vs Index: If dispWidth_numPts(1) < 2048 Then plotStep_ptNum = 1
            '   Else plotStep_ptNum is such that we plot between 2K and 4K points.
            ' X vx Y: plotStep is 1, and we plot every point
            plotStep_ptNum = IIf(xy, 1, (Int(.dispWidth_numPts / 2048) + 1))

            ' if writing to the screen instead of the Printer...
            If DrawingTo_PRINTER(g_form) Then
                gEndUser_PEN.Width = 0.5    ' .5/72" thick line
            Else
                gEndUser_PEN.Width = 1      ' 1/72" thick line
            End If

            ' ------------------------------------------------------
            ' Scan through points to plot.

            For ptNum = first_ptNum To last_ptNum Step plotStep_ptNum

                ' ---------------------------
                ' CALCULATE Y DATA

                ' Get Y data (attached to Display or Digitize Channel)
                If .cNum = NO_DIGI_CHAN Then
                    dataY = .dataArrayY.data(ptNum)
                Else
                    dataY = gDigiChan(.cNum).dataArrayY.data(ptNum)
                End If

                ' yCoord = (yEU + .plotOffset_yEU ) * .yCoord_Per_yEU
                nextPoint_yCoord = (dataY + .plotOffset_yEU) * .yCoord_Per_yEU

                ' make sure it does not overflow (else, we plot off the graph)
                If nextPoint_yCoord < .dispTop_yCoord Then nextPoint_yCoord = .dispTop_yCoord + 1
                If nextPoint_yCoord > .dispBot_yCoord Then nextPoint_yCoord = .dispBot_yCoord - 1

                ' ---------------------------
                ' CALCULATE X DATA

                ' if doing a XY display...
                If (xy) Then

                    ' Get X data if doing xy (attached to Display or Digitize Channel)
                    If .cNum = NO_DIGI_CHAN Then
                        dataX = .dataArrayX.data(ptNum)
                    Else
                        dataX = gDigiChan(.cNum + 1).dataArrayY.data(ptNum)
                    End If

                    ' debug
                    ' a = .xCoord_Per_xEU: b = .dispLeft_xEU: c = .dispLeft_xCoord

                    ' calc xCoord of point
                    nextPoint_xCoord = ((dataX - .dispLeft_xEU) * .xCoord_Per_xEU) + .dispLeft_xCoord

                    ' make sure it does not overflow (else, we plot off the graph)
                    If nextPoint_xCoord < .dispLeft_xCoord Then nextPoint_xCoord = .dispLeft_xCoord + 1
                    If nextPoint_xCoord > .dispRight_xCoord Then nextPoint_xCoord = .dispRight_xCoord - 1

                    ' else doing Y vs index
                Else
                    nextPoint_xCoord = ((ptNum - .dispLeft_ptNum) * .xCoord_Per_ptNum) _
                                        + .dispLeft_xCoord
                End If

                ' if we are to plot this point...
                If plotTheNextPoint Then

                    ' draw a little line segment between two plotted points
                    g_form.DrawLine(gEndUser_PEN, _
                        point_xCoord, point_yCoord, _
                        nextPoint_xCoord, nextPoint_yCoord)

                    ' plot little circles if there are less than 64 plotted points
                    If .dispWidth_numPts <= 64 And Not (xy) Then

                        g_form.DrawEllipse(gEndUser_PEN, _
                            nextPoint_xCoord - dotRadius_pixels, _
                            nextPoint_yCoord - dotRadius_pixels, _
                            dotRadius_pixels * 2, _
                            dotRadius_pixels * 2)

                    End If

                    ' otherwise, we are priming the pump, and will plot the next point
                Else
                    plotTheNextPoint = True
                End If

                'update the next point begin coordinate
                point_xCoord = nextPoint_xCoord
                point_yCoord = nextPoint_yCoord

            Next ptNum

        End With

EndOfPlotting:

        Set_gEndUser_PEN(kBlack, g_form)        ' Set pen to BLACK

        gEndUser_PEN.Width = 1                  ' 1/72" thick line

        Call Set_AutoRedraw_Property(False, g_form)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Plot_LastMarkerPos_OneDisp()
    '
    ' PURPOSE     Plots where the marker was
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Plot_LastMarkerPos_OneDisp(ByVal bNum As Short, ByVal dNum As Short, _
        ByRef thisBox As DispBoxParm, ByRef thisDisp As DispParm, ByRef g_form As System.Drawing.Graphics)

        ' -------------------------------------------------------------------
        ' Initialize

        Dim prevMarkerPos_ptNum As Integer
        Dim mkrNum As Short

        ' don't do anything in here if there are not displays in this DisplayBox.
        If (thisBox.numDisp <= 0) Then Exit Sub

        ' -------------------------------------------------------------------
        ' If we are only replotting where marker was, make sure this display
        ' contained a previous marker.

        For mkrNum = 0 To thisBox.numMkrs ' scan thru markers

            If (thisBox.prevMarkerPos_dNum(mkrNum) = dNum) Then

                ' calc point # of marker positions
                prevMarkerPos_ptNum = Translate_xEU_to_ptNum(bNum, dNum, thisBox.prevMarkerPos_xEU(mkrNum))

                ' plot where the marker was
                Call Plot_Segment_OneDisp(bNum, dNum, thisBox, thisDisp, _
                    (prevMarkerPos_ptNum - 3), (prevMarkerPos_ptNum + 3), g_form)

            End If

        Next mkrNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Plot_FullScreen_OneDisp()
    '
    ' PURPOSE     Plot entire waveform within 1 Display.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Plot_FullScreen_OneDisp(ByVal bNum As Short, ByVal dNum As Short, _
        ByRef thisBox As DispBoxParm, ByRef thisDisp As DispParm, ByRef g_form As System.Drawing.Graphics)

        Dim first_ptNum, last_ptNum, numValidPts As Integer
        Dim ptsPerXunit, firstPt_Xunits As Double

        ' if XY Display..
        If (thisDisp.displayType = kXYDisplay) Then

            ' get the # of data pts and the data's sample rate (for the Y data)
            Call GetDataParams(bNum, dNum, ptsPerXunit, numValidPts, firstPt_Xunits)

            ' plot all points inside the data buffer
            first_ptNum = 0
            last_ptNum = numValidPts - 1

            ' else Y vs INDEX Display...
        Else

            ' plot from left to right edge of display
            first_ptNum = thisDisp.dispLeft_ptNum
            last_ptNum = thisDisp.dispRight_ptNum

        End If

        ' Plot entire display, from left to right edges
        Call Plot_Segment_OneDisp(bNum, dNum, thisBox, thisDisp, first_ptNum, last_ptNum, g_form)

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Grid_Draw_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by
    '             the end user.
    '
    '             Draw the grid in all displays of one display box.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Grid_Draw_OneBox(ByVal bNum As Short, ByRef g_form As System.Drawing.Graphics)

        Dim dNum As Short

        For dNum = 0 To (gBox(bNum).numDisp - 1)

            ' draw the grid for 1 display
            Call Grid_Draw_OneDisp(bNum, dNum, gBox(bNum), gBox(bNum).disp(dNum), g_form)

        Next dNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Plot_FullScreen_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '            Plot data in all displays of one display box
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Plot_FullScreen_OneBox(ByVal bNum As Short, ByRef g_form As System.Drawing.Graphics)

        Dim dNum As Short

        For dNum = 0 To (gBox(bNum).numDisp - 1)

            Call Plot_FullScreen_OneDisp(bNum, dNum, gBox(bNum), gBox(bNum).disp(dNum), g_form)

        Next dNum

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    Grid_Draw_OneDisp()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Draw grid in one display.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Grid_Draw_OneDisp(ByVal bNum As Short, ByVal dNum As Short, _
        ByRef thisBox As DispBoxParm, ByRef thisDisp As DispParm, ByRef g_form As System.Drawing.Graphics)

        Dim tick_yCoord, tick_right_xCoord, tick_left_xCoord As Integer
        Dim halfTickWidth_xCoord, numTickSpaces_vert, xCoord As Integer
        Dim tick_xCoord, Ticks, tick_bottom_yCoord, tick_top_yCoord As Integer
        Dim halfTickWidth_yCoord, numTickSpaces_horiz, yCoord, i As Integer

        Dim ticksPerGrid_vert As Integer      ' # of tics per vert grid, e.g. 3 (-1=autoselect)
        Dim ticksPerGrid_horiz As Integer     ' # of tics per horiz grid, e.g. 3 (-1=autoselect)
        Dim tickSpacing_xCoord As Double
        Dim tickSpacing_yCoord As Double

        ' don't do anything in here if there are not displays in this DisplayBox.
        If (thisBox.numDisp <= 0) Then Exit Sub

        ' if we don't show a grid in this display, bail out
        If (thisDisp.showGrid = False) Then Exit Sub

        ' turn on AutoRedraw
        Call Set_AutoRedraw_Property(True, g_form)

        ' select grid line style
        Call Set_gEndUser_PEN(kGray, g_form)

        ' if writing to the screen instead of the Printer...
        If DrawingTo_PRINTER(g_form) Then
            gEndUser_PEN.Width = 0.5    ' .5/72" thick line (.007")
        Else
            gEndUser_PEN.Width = 1      ' 1/72" thick line
        End If

        With thisDisp

            ' -----------------------------------------------------
            ' Draw horiz grid lines.

            For i = 1 To .numGridLines_horiz_calc

                ' calc yCoord of horiz line
                yCoord = .gridHeight_yCoord * i + .dispTop_yCoord

                ' if this is the center line, then draw tick marks
                If (i = (Int(.numGridLines_horiz_calc / 2) + 1)) Then

                    ' now add little tick marks along this horiz line
                    numTickSpaces_horiz = ((.numGridLines_vert_calc + 1) * (.ticksPerGrid_horiz_calc + 1))
                    halfTickWidth_yCoord = CrtPix_to_GraphicsPix(2, g_form) ' 2 pixel tick marks
                    tick_top_yCoord = yCoord - halfTickWidth_yCoord
                    tick_bottom_yCoord = yCoord + halfTickWidth_yCoord + CrtPix_to_GraphicsPix(2, g_form)
                    tickSpacing_xCoord = (.dispRight_xCoord - .dispLeft_xCoord) / numTickSpaces_horiz

                    For Ticks = 1 To (numTickSpaces_horiz - 1)
                        tick_xCoord = tickSpacing_xCoord * Ticks + .dispLeft_xCoord
                        g_form.DrawLine(gEndUser_PEN, tick_xCoord, tick_top_yCoord, tick_xCoord, tick_bottom_yCoord)
                    Next Ticks

                End If

                ' draw grid line
                g_form.DrawLine(gEndUser_PEN, .dispLeft_xCoord, yCoord, .dispRight_xCoord, yCoord)

            Next i

            ' -----------------------------------------------------
            ' Draw vertical grid lines

            For i = 1 To .numGridLines_vert_calc

                ' calc xCoord of vertical line
                xCoord = .gridWidth_xCoord * i + .dispLeft_xCoord

                ' if this is the center line, then add tick marks
                If (i = (Int(.numGridLines_vert_calc / 2) + 1)) Then

                    ' now add little tick marks along this vertical line
                    numTickSpaces_vert = ((.numGridLines_horiz_calc + 1) * (.ticksPerGrid_vert_calc + 1))
                    halfTickWidth_xCoord = CrtPix_to_GraphicsPix(2, g_form) ' 2 pixel tick mark
                    tick_left_xCoord = xCoord - halfTickWidth_xCoord
                    tick_right_xCoord = xCoord + halfTickWidth_xCoord + CrtPix_to_GraphicsPix(2, g_form)
                    tickSpacing_yCoord = (.dispBot_yCoord - .dispTop_yCoord) / numTickSpaces_vert

                    For Ticks = 1 To (numTickSpaces_vert - 1)
                        tick_yCoord = tickSpacing_yCoord * Ticks + .dispTop_yCoord
                        g_form.DrawLine(gEndUser_PEN, tick_left_xCoord, tick_yCoord, tick_right_xCoord, tick_yCoord)
                    Next Ticks

                End If

                ' draw grid line
                g_form.DrawLine(gEndUser_PEN, xCoord, .dispTop_yCoord, xCoord, .dispBot_yCoord)

            Next i

        End With

        Call Set_gEndUser_PEN(kBlack, g_form)               ' go back to drawing black
        gEndUser_PEN.Width = 1                              ' 1/72" thick line
        Call Set_AutoRedraw_Property(False, g_form)         ' turn AutoRedraw off

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    hScale_DrawLowerLabel_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Draw Lower hScale label at base of DisplayBox.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub hScale_DrawLowerLabel_OneBox(ByVal bNum As Short, _
        ByRef thisBox As DispBoxParm, ByRef g_form As System.Drawing.Graphics)

        Dim xEU, xCoord_double As Double
        Dim xEU_str As String
        Dim textWidth_xCoord, prevLabelRightEdge_xCoord, xCoord_int, i, Pen_xCoord, Pen_yCoord As Integer

        ' 4/72th inches = 0.05" inches
        Dim hLABEL_SPACE As Short = CrtPix_to_GraphicsPix(4, g_form)

        With thisBox

            ' don't do anything in here if there are not displays in this DisplayBox.
            If (.numDisp <= 0) Then Exit Sub

            Call Set_AutoRedraw_Property(True, g_form)

            ' -----------------------------------------------------
            ' Fill the entire LowerLabel region w/ white.

            Clear_Rectangle((.dispBoxLeft_xCoord + 1), (.dispBoxBot_yCoord - 1), _
                (.dispBoxRight_xCoord - 1), (.hLowerLabel_Top_yCoord + 1), g_form)

            ' -----------------------------------------------------
            ' Draw hScale (e.g. "0   .1   .2   .3   .4 secs"

            prevLabelRightEdge_xCoord = -(hLABEL_SPACE * 2) + .dispBoxLeft_xCoord

            ' select black line style
            Call Set_gEndUser_PEN(kBlack, g_form)

            For i = 0 To .disp(TOP_DISPLAY_NUM).numGridLines_vert_calc + 1

                ' calc xCoord of vertical line
                xCoord_double = (.disp(TOP_DISPLAY_NUM).gridWidth_xCoord * i) + .dispBoxLeft_xCoord
                xCoord_int = xCoord_double

                ' get xEU value for this grid line
                With thisBox.disp(TOP_DISPLAY_NUM)

                    ' calc xEU value
                    xEU = ((xCoord_double - .dispLeft_xCoord) / .xCoord_Per_xEU) + .dispLeft_xEU

                    '  if the display width is > .1, then round off to nearest .001
                    If .dispWidth_xEU > 0.1 Then
                        xEU = Int((xEU * 1000.0#) + 0.5) / 1000.0#
                    End If

                End With

                xEU_str = NumToStr(xEU)

                ' get width of string, in pixels
                textWidth_xCoord = g_form.MeasureString(xEU_str, gEndUser_FONT).Width

                ' move pen
                If i = 0 Then ' if we are at the left edge, then push right
                    Pen_xCoord = xCoord_int + hLABEL_SPACE
                Else
                    Pen_xCoord = xCoord_int - (textWidth_xCoord / 2)
                End If

                ' if we are hitting the previous label, then move right some
                If (Pen_xCoord < (prevLabelRightEdge_xCoord + (hLABEL_SPACE * 3))) Then
                    Pen_xCoord = (prevLabelRightEdge_xCoord + (hLABEL_SPACE * 3))
                End If

                ' if we over shot the grid line, then skip this label
                If (Pen_xCoord > xCoord_int + hLABEL_SPACE + 1) Then GoTo NextGrid

                ' set the upper left coord of the printed label
                Pen_yCoord = .hLowerLabel_Top_yCoord + vSPACE_yPix(g_form)

                ' remember the right edge of the recently drawn label
                prevLabelRightEdge_xCoord = Pen_xCoord + textWidth_xCoord

                ' print the xEU scale value
                g_form.DrawString(xEU_str, gEndUser_FONT, Brushes.Black, Pen_xCoord, Pen_yCoord)

NextGrid:

            Next i

            Call Set_AutoRedraw_Property(False, g_form)

        End With

    End Sub


    ' -------------------------------------------------------------------------
    ' FUNCTION    CrtPix_to_GraphicsPix()
    '
    ' PURPOSE     Converts 72dpi crtPixel length to corresponding # of graphicsPixels 
    '             use by GRAPHICS device. If drawing to screen, we return the
    '             same value. If drawing to printer, we convert the 72dpi crtPixel
    '             value to a corresponding length in higher pixel system 
    '             (e.g. 300 or 1200 dpi).  For example 36 crtPixels corresponds
    '             to .5", and on a 1200 dpi printer, we would return 600 graphicsPixels.
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Function CrtPix_to_GraphicsPix(ByVal crtPixels As Single, _
        ByRef g_form As System.Drawing.Graphics) As Integer

        If DrawingTo_PRINTER(g_form) Then       ' if drawing to a printer
            CrtPix_to_GraphicsPix = g_form.DpiX * crtPixels / 72.0
        Else                                    ' else, we are drawing to a screen
            CrtPix_to_GraphicsPix = crtPixels   ' use screen pixels
        End If

        '                                         make sure we don't go below 1
        If (CrtPix_to_GraphicsPix < 1) Then CrtPix_to_GraphicsPix = 1

    End Function


    ' -------------------------------------------------------------------------
    ' FUNCTION    Borders_Draw_OneBox()
    '
    ' PURPOSE     This is a low level routine that is typically not used by the end user.
    '
    '             Draw borders in one DisplayBox (i.e. enclosing rect, lines between Displays,
    '             hUpperLabelBox border, vLabelBox border, hLowerLabelBox border).
    '
    ' HISTORY     6/5/02 Updated for Visual Basic 7, gsw
    ' -------------------------------------------------------------------------
    Public Sub Borders_Draw_OneBox(ByVal bNum As Short, ByRef g_form As System.Drawing.Graphics)

        Dim dNum As Short

        ' draw black
        Call Set_gEndUser_PEN(kBlack, g_form)

        With gBox(bNum)

            Call Set_AutoRedraw_Property(True, g_form)

            ' draw line below the Upper Horizontal Label region
            g_form.DrawLine(gEndUser_PEN, .dispBoxLeft_xCoord, .hUpperLabel_Bot_yCoord, _
                    .dispBoxRight_xCoord, .hUpperLabel_Bot_yCoord)

            ' draw line above the Lower Horizontal Label region
            g_form.DrawLine(gEndUser_PEN, .dispBoxLeft_xCoord, .hLowerLabel_Top_yCoord, _
                .dispBoxRight_xCoord, .hLowerLabel_Top_yCoord)

            ' draw line to left of the Vertical Label region
            g_form.DrawLine(gEndUser_PEN, .vLabelBoxLeft_xCoord, .hUpperLabel_Bot_yCoord, _
                .vLabelBoxLeft_xCoord, .hLowerLabel_Top_yCoord)

            ' double thick line 
            If DrawingTo_PRINTER(g_form) Then
                gEndUser_PEN.Width = 4  ' (4/72th of an inch)
            Else
                gEndUser_PEN.Width = 2  ' (2/72th of an inch)
            End If

            ' draw line under each Display plot region
            For dNum = 0 To (.numDisp - 2)

                g_form.DrawLine(gEndUser_PEN, _
                    .disp(dNum).dispLeft_xCoord, _
                    .disp(dNum).dispBot_yCoord, _
                    .dispBoxRight_xCoord, _
                    .disp(dNum).dispBot_yCoord)

            Next dNum

            ' thicker line (2/72th of an inch)
            gEndUser_PEN.Width = 2

            ' draw a box around the set of plot regions (i.e. the "DisplayBox")
            Dim dispBox_rect As New Rectangle( _
                .dispBoxLeft_xCoord, _
                .dispBoxTop_yCoord, _
                .dispBoxRight_xCoord - .dispBoxLeft_xCoord, _
                .dispBoxBot_yCoord - .dispBoxTop_yCoord)

            g_form.DrawRectangle(gEndUser_PEN, dispBox_rect)

            ' thin line (1/72th of an inch)
            gEndUser_PEN.Width = 1

            Call Set_AutoRedraw_Property(False, g_form)

        End With

    End Sub

End Module